@@ -3,28 +3,27 @@ Import this only from your test cases
 
 Example:
 
-    from runez.conftest import cli, isolated_log_setup, temp_folder
+    from runez.conftest import cli, temp_folder
 """
 
 import logging
 import os
 import re
 import sys
-import tempfile
 
 import _pytest.logging
 import pytest
 
 import runez.config
+import runez.system
 from runez.colors import ActivateColors
 from runez.file import TempFolder
 from runez.logsetup import LogManager
 from runez.render import Header
-from runez.system import _R, CaptureOutput, DEV, Slotted, TempArgv, TrackedOutput
-from runez.system import flattened, LOG, quoted, short, stringified, UNSET
+from runez.system import _R, CaptureOutput, DEV, flattened, LOG, quoted, short, Slotted, stringified, TempArgv, TrackedOutput, UNSET
 
 try:
-    from click import BaseCommand as _ClickCommand
+    from click import Command as _ClickCommand
     from click.testing import CliRunner as _CliRunner
 
 except ImportError:  # pragma: no cover, click used only if installed
@@ -32,8 +31,6 @@ except ImportError:  # pragma: no cover, click used only if installed
     _CliRunner = None
 
 
-TMP = tempfile.gettempdir()
-
 # Set DEBUG logging level when running tests, makes sure LOG.debug() calls get captured (for inspection in tests)
 logging.root.setLevel(logging.DEBUG)
 
@@ -45,7 +42,7 @@ if sys.argv and "pycharm" in sys.argv[0].lower():  # pragma: no cover, ignore Py
         sys.argv[0] = pt
 
 # Set logsetup defaults to stable/meaningful for pytest runs
-LogManager.override_spec(timezone="UTC", tmp=TMP, locations=[os.path.join("{tmp}", "{basename}")])
+LogManager.override_spec(timezone="UTC", locations=["{tmp}/{basename}"])
 
 
 def exception_raiser(exc=Exception):
@@ -62,13 +59,11 @@ def exception_raiser(exc=Exception):
     Returns:
         (callable): Function that will raise given exception
     """
+
     def _raise(*_, **__):
         if isinstance(exc, str):
             raise Exception(exc)
 
-        if isinstance(exc, type) and issubclass(exc, BaseException):
-            raise exc()
-
         raise exc
 
     return _raise
@@ -86,7 +81,7 @@ def patch_env(monkeypatch, clear=True, uppercase=True, **values):
         values = {k.upper(): v for k, v in values.items()}
 
     if clear:
-        for k in os.environ.keys():
+        for k in os.environ:
             if k not in values:
                 monkeypatch.delenv(k)
 
@@ -111,6 +106,9 @@ class IsolatedLogSetup:
         """
         self.adjust_tmp = adjust_tmp
         self.temp_folder = None
+        self.abort_exception = None
+        self.old_cwd = None
+        self.old_env_vars = None
 
     def __enter__(self):
         WrappedHandler.isolation += 1
@@ -125,6 +123,11 @@ class IsolatedLogSetup:
             self.temp_folder = TempFolder()
             LogManager.spec.tmp = self.temp_folder.__enter__()
 
+        else:
+            self.abort_exception = runez.system.AbortException
+            self.old_cwd = os.getcwd()
+
+        self.old_env_vars = dict(os.environ)
         return self.temp_folder and self.temp_folder.tmp_folder
 
     def __exit__(self, *_):
@@ -134,9 +137,24 @@ class IsolatedLogSetup:
         logging.root.handlers = self.old_handlers
         WrappedHandler.isolation -= 1
         LogManager.reset()
+        # Restore env vars as they were
+        for k in os.environ:
+            if k not in self.old_env_vars:
+                del os.environ[k]
+
+        for k, v in self.old_env_vars.items():
+            if os.environ.get(k) != v:
+                os.environ[k] = v
+
         if self.temp_folder:
             self.temp_folder.__exit__()
 
+        else:
+            runez.system.AbortException = self.abort_exception
+            if self.old_cwd and os.path.isdir(self.old_cwd):
+                # Edge case: some tests 'cd' to a now-deleted temp folder (side effect)
+                os.chdir(self.old_cwd)
+
 
 @pytest.fixture
 def cli():
@@ -162,9 +180,6 @@ def cli():
         yield ClickRunner(context=context)
 
 
-# This just allows to get auto-complete to work in PyCharm
-cli = cli  # type: ClickRunner
-
 # Comes in handy for click apps with only one main entry point
 cli.default_main = None
 
@@ -172,13 +187,6 @@ cli.default_main = None
 cli.context = TempFolder
 
 
-@pytest.fixture
-def isolated_log_setup():
-    """Log settings restored"""
-    with IsolatedLogSetup() as tmp:
-        yield tmp
-
-
 @pytest.fixture
 def logged():
     with CaptureOutput(seed_logging=True) as logged:
@@ -191,11 +199,6 @@ def temp_folder():
         yield tmp
 
 
-# This just allows to get auto-complete to work in PyCharm
-logged = logged  # type: TrackedOutput
-temp_folder = temp_folder  # type: str
-
-
 class WrappedHandler(_pytest.logging.LogCaptureHandler):
     """pytest aggressively imposes its own capture, this allows to impose our capture where applicable"""
 
@@ -252,7 +255,7 @@ class ClickRunner:
 
     args: list = None  # Arguments used in last run()
     exit_code: int = None  # Exit code of last run()
-    logged: TrackedOutput = None  # Captured log from last run()
+    logged: TrackedOutput  # Captured log from last run()
     main: callable = None  # Optional, override default_main for this runner instance
     trace: bool = None  # Optional, enable trace logging for this runner instance
 
@@ -304,7 +307,7 @@ class ClickRunner:
                 if r.failed:
                     msg = "%s --help failed" % runez.short(script)
                     logging.error("%s\n%s", msg, r.full_output)
-                    assert False, msg
+                    raise AssertionError(msg)
 
     def run(self, *args, exe=None, main=UNSET, trace=UNSET):
         """
@@ -413,20 +416,17 @@ class ClickRunner:
             elif expected in contents:
                 i = contents.index(expected)
                 pre = short(contents[:i], size=32)
-                post = short(contents[i + len(expected):], size=32)
+                post = short(contents[i + len(expected) :], size=32)
                 return Match(c, expected, pre=pre, post=post)
 
     def expect_messages(self, *expected, **kwargs):
         for message in expected:
             if message[0] == "!":
                 m = self.match(message[1:], **kwargs)
-                if m:
-                    assert False, "Unexpected match in output: %s" % m
-
+                assert not m, "Unexpected match in output: %s" % m
             else:
                 m = self.match(message, **kwargs)
-                if not m:
-                    assert False, "Not seen in output: %s" % message
+                assert m, "Not seen in output: %s" % message
 
     def expect_success(self, args, *expected, **kwargs):
         spec = RunSpec()
@@ -488,19 +488,14 @@ class ClickRunner:
 
             return result
 
-        if isinstance(main, str):
-            script = self._resolved_script(main)
-            if not script:
-                assert False, "Can't find script '%s', invalid main" % script
-
-            r = runez.run(sys.executable, script, *args, fatal=False)
-            return ClickWrapper(r.output, r.error, r.exit_code, r.exc_info)
-
-        assert False, "Can't invoke invalid main: %s" % main
+        assert isinstance(main, str), "Can't invoke invalid main: %s" % main
+        script = self._resolved_script(main)
+        assert script, "Can't find script '%s', invalid main" % main
+        r = runez.run(sys.executable, script, *args, fatal=False)
+        return ClickWrapper(r.output, r.error, r.exit_code, r.exc_info)
 
 
 class RunSpec(Slotted):
-
     __slots__ = ["stdout", "stderr", "regex"]
 
     def _get_defaults(self):
