@@ -66,9 +66,9 @@ def copy(source, destination, ignore=None, overwrite=True, fatal=True, logger=UN
         destination (str | Path | None): Destination file or folder
         ignore (callable | list | str | None): Names to be ignored
         overwrite (bool | None): True: replace existing, False: fail if destination exists, None: no destination check
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -80,9 +80,9 @@ def delete(path, fatal=True, logger=UNSET, dryrun=UNSET):
     """
     Args:
         path (str | Path | None): Path to file or folder to delete
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -110,9 +110,9 @@ def ensure_folder(path, clean=False, fatal=True, logger=UNSET, dryrun=UNSET):
     Args:
         path (str | Path | None): Path to file or folder
         clean (bool): True: If True, ensure folder is clean (delete any file/folder it may have)
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, >=1: successfully done, 0: was no-op, -1: failed
@@ -130,7 +130,7 @@ def ensure_folder(path, clean=False, fatal=True, logger=UNSET, dryrun=UNSET):
             cleaned += delete(os.path.join(path, fname), fatal=fatal, logger=None, dryrun=dryrun)
 
         if cleaned:
-            msg = "%s from %s" % (_R.plural(cleaned, "file"), short(path))
+            msg = "%s from %s" % (_R.lc.rm.plural(cleaned, "file"), short(path))
             if not _R.hdry(dryrun, logger, "clean %s" % msg):
                 _R.hlog(logger, "Cleaned %s" % msg)
 
@@ -182,7 +182,7 @@ def ini_to_dict(path, keep_empty=False, fatal=False, logger=False):
     Args:
         path (str | Path | None): Path to file to parse
         keep_empty (bool): If True, keep definitions with empty values
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
 
     Returns:
@@ -213,11 +213,27 @@ def ini_to_dict(path, keep_empty=False, fatal=False, logger=False):
                 section[key] = value
 
     if not keep_empty:
-        result = dict((k, v) for k, v in result.items() if k and v)
+        result = {k: v for k, v in result.items() if k and v}
 
     return result
 
 
+def is_subfolder(path, root_folder):
+    """
+    Args:
+        path (str | Path): Path to file or folder
+        root_folder (str | Path): Path to folder to consider as root
+
+    Returns:
+        (bool): True if 'path' is a sub-folder of 'root_folder'
+    """
+    if path and root_folder:
+        abs_path = os.path.abspath(path)
+        abs_root = os.path.abspath(root_folder)
+        prefix = os.path.commonprefix([abs_path, abs_root])
+        return prefix == abs_root
+
+
 def is_younger(path, age, default=False):
     """
     Args:
@@ -263,14 +279,15 @@ def parent_folder(path, base=None):
     return path and os.path.dirname(resolved_path(path, base=base))
 
 
-def readlines(path, first=None, errors="ignore", fatal=False, logger=False):
+def readlines(path, first=None, errors="ignore", fatal=False, logger=False, transform=str.rstrip):
     """
     Args:
         path (str | Path | None): Path to file to read lines from
         first (int | None): Return only the 'first' lines when specified
         errors (str | None): Optional string specifying how encoding errors are to be handled
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
+        transform (callable | None): Optional callable to transform each line
 
     Yields:
         (str): Lines read, newlines and trailing spaces stripped
@@ -284,7 +301,10 @@ def readlines(path, first=None, errors="ignore", fatal=False, logger=False):
                 if first == 0:
                     return
 
-                yield decode(line).rstrip()
+                line = decode(line)
+                if transform:
+                    line = transform(line)
+                yield line
                 first -= 1
 
     except Exception as e:
@@ -321,12 +341,12 @@ def move(source, destination, overwrite=True, fatal=True, logger=UNSET, dryrun=U
     """Move `source` -> `destination`
 
     Args:
-        source (str | None): Source file or folder
-        destination (str | None): Destination file or folder
+        source (str | Path | None): Source file or folder
+        destination (str | Path | None): Destination file or folder
         overwrite (bool | None): True: replace existing, False: fail if destination exists, None: no destination check
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -342,9 +362,9 @@ def symlink(source, destination, must_exist=True, overwrite=True, fatal=True, lo
         destination (str | Path | None): Destination file or folder
         must_exist (bool): If True, verify that source does indeed exist
         overwrite (bool | None): True: replace existing, False: fail if destination exists, None: no destination check
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -360,9 +380,9 @@ def compress(source, destination, arcname=UNSET, ext=None, overwrite=True, fatal
         arcname (str | None): Name of subfolder in archive (default: source basename)
         ext (str | None): Extension determining compression (default: extension of given 'source' file)
         overwrite (bool | None): True: replace existing, False: fail if destination exists, None: no destination check
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -398,9 +418,9 @@ def decompress(source, destination, ext=None, overwrite=True, simplify=False, fa
         ext (str | None): Extension determining compression (default: extension of given 'source' file)
         overwrite (bool | None): True: replace existing, False: fail if destination exists, None: no destination check
         simplify (bool): If True and source has only one sub-folder, extract that one sub-folder to destination
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -424,7 +444,7 @@ class TempFolder:
         """
         Args:
             anchor (bool): If True, short-ify paths relative to used temp folder
-            dryrun (bool): Optionally override current dryrun setting
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
             follow (bool): If True, change working dir to temp folder (and restore)
         """
         self.anchor = anchor
@@ -465,9 +485,9 @@ def touch(path, fatal=True, logger=UNSET, dryrun=UNSET):
 
     Args:
         path (str | Path | None): Path to file to touch
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -481,9 +501,9 @@ def write(path, contents, fatal=True, logger=UNSET, dryrun=UNSET):
     Args:
         path (str | Path | None): Path to file
         contents (str | bytes | None): Contents to write (only touch file if None)
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -492,12 +512,8 @@ def write(path, contents, fatal=True, logger=UNSET, dryrun=UNSET):
         return 0
 
     path = resolved_path(path)
-    byte_size = _R.represented_bytesize(len(contents)) if contents else ""
-
-    def dryrun_msg():
-        return "%s %s" % ("write %s to" % byte_size if byte_size else "touch", short(path))
-
-    if _R.hdry(dryrun, logger, dryrun_msg):
+    short_path = short(path)
+    if _R.hdry(dryrun, logger, "%s %s" % ("write" if contents else "touch", short_path)):
         return 1
 
     ensure_folder(parent_folder(path), fatal=fatal, logger=None, dryrun=dryrun)
@@ -510,11 +526,11 @@ def write(path, contents, fatal=True, logger=UNSET, dryrun=UNSET):
             else:
                 fh.write(contents)
 
-        _R.hlog(logger, "%s %s" % ("Wrote %s to" % byte_size if byte_size else "Touched", short(path)))
+        _R.hlog(logger, "%s %s" % ("Wrote" if contents else "Touched", short_path))
         return 1
 
     except Exception as e:
-        return abort("Can't write to %s" % short(path), exc_info=e, return_value=-1, fatal=fatal, logger=logger)
+        return abort("Can't write to %s" % short_path, exc_info=e, return_value=-1, fatal=fatal, logger=logger)
 
 
 def _copy(source, destination, ignore=None):
@@ -592,7 +608,13 @@ def _untar(source, destination, simplify):
     with TempFolder():
         extracted_source = to_path(source.name)
         with tarfile.open(source) as fh:
-            fh.extractall(extracted_source)
+            for member in fh.getmembers():
+                # See https://www.trellix.com/en-us/about/newsroom/stories/research/tarfile-exploiting-the-world.html
+                member_path = os.path.join(extracted_source, member.name)
+                if not is_subfolder(member_path, extracted_source):  # pragma: no cover, don't have an exploit sample handy
+                    raise Exception("Attempted Path Traversal in Tar File")
+
+            fh.extractall(extracted_source, filter="fully_trusted")  # noqa: S202, taken care of
 
         _move_extracted(extracted_source, destination, simplify)
 
@@ -606,7 +628,7 @@ def _unzip(source, destination, simplify):
     with TempFolder():
         extracted_source = to_path(source.name)
         with ZipFile(source) as fh:
-            fh.extractall(extracted_source)
+            fh.extractall(extracted_source)  # noqa: S202, not a tarfile
 
         _move_extracted(extracted_source, destination, simplify)
 
@@ -614,7 +636,7 @@ def _unzip(source, destination, simplify):
 def _zip(source, destination, arcname, fh=None):
     """Effective zip, behaving like tar+gzip for consistency"""
     if fh is None:
-        from zipfile import ZipFile, ZIP_DEFLATED
+        from zipfile import ZIP_DEFLATED, ZipFile
 
         source = to_path(source).absolute()
         destination = to_path(destination).absolute()
@@ -633,13 +655,13 @@ def _file_op(source, destination, func, overwrite, fatal, logger, dryrun, must_e
     """Call func(source, destination)
 
     Args:
-        source (str | None): Source file or folder
-        destination (str | None): Destination file or folder
+        source (str | Path | None): Source file or folder
+        destination (str | Path | None): Destination file or folder
         func (callable): Implementation function
         overwrite (bool | None): True: replace existing, False: fail if destination exists, None: no destination check
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
         must_exist (bool): If True, verify that source does indeed exist
         ignore (callable | list | str | None): Names to be ignored
         **extra: Passed-through to 'func'
@@ -647,6 +669,8 @@ def _file_op(source, destination, func, overwrite, fatal, logger, dryrun, must_e
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
     """
+    source = str(source)  # Using str to remain compatible with py3.6 for a while longer
+    destination = str(destination)
     if not source or not destination or source == destination:
         return 0
 
