@@ -26,12 +26,13 @@ import functools
 import json
 import os
 import re
+import sys
 import urllib.parse
 from pathlib import Path
+from typing import ClassVar
 
 from runez.file import checksum, decompress, delete, ensure_folder, TempFolder, to_path
-from runez.logsetup import LogManager
-from runez.system import _R, abort, find_caller, short, stringified, SYS_INFO, UNSET
+from runez.system import _R, abort, DEV, find_caller, joined, short, stringified, SYS_INFO, UNSET
 
 
 def urljoin(base, url):
@@ -49,6 +50,123 @@ def urljoin(base, url):
     return urllib.parse.urljoin(base, url, allow_fragments=False)
 
 
+class CacheWrapper:
+    """
+    Simple cache wrapper, intended to provide a convenient way of caching REST calls via pypi module diskcache for example
+    By default, ~/.cache/{program_name} is used
+
+    Example usage:
+        cache = RestClient.std_diskcache(size_limit="4g")
+        client = RestClient(url, cache=cache)
+    """
+
+    base_location = "~/.cache/{program_name}"
+    cachable_methods = ("GET", "POST")  # By default, cache only these REST methods
+    default_expire = "1h"  # 1 hour
+    size_limit = "2g"  # 2 GB
+
+    __dev_suffix = "-dev"  # Override to None to disable distinguishing dev runs from non-dev runs
+
+    @classmethod
+    def resolved_base_location(cls, base_location=UNSET):
+        if base_location is UNSET:
+            base_location = cls.base_location
+
+        if base_location:
+            base_location = base_location.format(program_name=SYS_INFO.program_name)
+            return os.path.expanduser(base_location)
+
+    @classmethod
+    def cache_base_path(cls, base_location=UNSET, suffix=None, dev_suffix=UNSET):
+        """
+        Args:
+            base_location (str | None): Base location to use, {program_name}
+            suffix (str | None): Optional suffix for location basename to add
+            dev_suffix (str | None): Suffix to add for detected development runs (from a dev venv)
+
+        Returns:
+            (str): Fully resolved location, example: ~/.cache/my-program-py39-dev
+        """
+        path = cls.resolved_base_location(base_location)
+        if path:
+            if suffix:
+                path += "-%s" % suffix
+
+            if dev_suffix is UNSET:
+                dev_suffix = cls.__dev_suffix
+
+            if dev_suffix and DEV.venv_folder:
+                # Allows to distinguish development runs (dev and "prod" runs won't use the same cache)
+                path += dev_suffix
+
+        return path
+
+    def __init__(self, cache_backend, base_location, default_expire, size_limit):
+        """
+        Args:
+            cache_backend: Underlying backend to use
+            base_location (str | None): Cache base location
+            default_expire (int | float | None): Default expiration time in seconds
+            size_limit (int | None): Max size in bytes for this cache
+        """
+        self.base_location = base_location
+        self.default_expire = _R.lc.rm.to_seconds(default_expire)
+        self.size_limit = _R.lc.rm.to_bytesize(size_limit)
+        self.cache_backend = cache_backend
+
+    @staticmethod
+    def cache_key(full_url, params=None):
+        """
+        Args:
+            full_url (str): Full URL to remote asset
+            params (dict | None): Optional REST query params
+
+        Returns:
+            (str): Key to use to represent this remote asset in the cache
+        """
+        key = full_url
+        if params:
+            key += "?%s" % urllib.parse.urlencode(params)
+
+        return key
+
+    def is_cachable_method(self, method):
+        """
+        Args:
+            method (str): Should this method be cached (REST methods: DELETE, GET, HEAD, POST, PURGE, PUT)
+
+        Returns:
+            (bool): True if this REST method call should be cached
+        """
+        return method in self.cachable_methods
+
+    def delete(self, cache_key):
+        """
+        Args:
+            cache_key (str): Key to delete from cache
+        """
+        self.cache_backend.delete(cache_key)
+
+    def get(self, cache_key):
+        """
+        Args:
+            cache_key (str): Key to lookup from this cache
+        """
+        return self.cache_backend.get(cache_key)
+
+    def set(self, cache_key, data, expire=UNSET):
+        """
+        Args:
+            cache_key (str): Key to store `data` under
+            data: Data to store
+            expire (int | float | None): Expiration time in seconds
+        """
+        if expire is UNSET:
+            expire = self.default_expire
+
+        return self.cache_backend.set(cache_key, data, expire=expire)
+
+
 class ForbiddenHttpError(Exception):
     """Raised to signify test setup prevented a remote call"""
 
@@ -86,6 +204,7 @@ class GlobalHttpCalls:
     @classmethod
     def allowed(cls, func):
         """Decorator for test_ functions, temporarily allows external requests"""
+
         @functools.wraps(func)
         def inner(*args, **kwargs):
             with GlobalHttpCalls(True):
@@ -96,6 +215,7 @@ class GlobalHttpCalls:
     @classmethod
     def forbidden(cls, func):
         """Decorator for test_ functions, temporarily forbids external requests"""
+
         @functools.wraps(func)
         def inner(*args, **kwargs):
             with GlobalHttpCalls(False):
@@ -160,7 +280,7 @@ class DataState:
     def wrapped(cls, dryrun, data, json, files, filepaths):
         """
         Args:
-            dryrun (bool): Optionally override current dryrun setting
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
             data (dict | tuple | bytes | file | None): Data to send in the body
             json: (optional) json to send in the body
             files (dict | None): File-like-objects for multipart encoding upload.
@@ -186,7 +306,6 @@ class DataState:
 
 
 class MockResponse:
-
     def __init__(self, status_code, content):
         self.status_code = status_code
         if content is not None and not isinstance(content, bytes):
@@ -206,7 +325,6 @@ class MockResponse:
 
 
 class MockedHandlerStack:
-
     def __init__(self):
         self.handler = None
         self.ms = None
@@ -252,7 +370,7 @@ class MockedHandlerStack:
     def response_for_url(self, method, url):
         spec = self.specs.get(url)
         if spec is None:
-            return MockResponse(404, dict(message="Default status code 404"))
+            return MockResponse(404, {"message": "Default status code 404"})
 
         if isinstance(spec, BaseException):
             raise spec
@@ -276,8 +394,7 @@ class MockedHandlerStack:
 
 
 class MockCentral:
-
-    _stacks = {}
+    _stacks: ClassVar = {}
 
     @classmethod
     def get_stack(cls, handler, key):
@@ -398,7 +515,7 @@ class RestResponse:
             if msg:
                 return msg
 
-        except Exception:  # nosec
+        except Exception:  # noqa: S110
             pass
 
         return self.text
@@ -460,10 +577,6 @@ class RestHandler:
 
         return MockWrapper(cls, base_url, specs)
 
-    @classmethod
-    def is_usable(cls):
-        """Is this handler currently usable"""
-
     @classmethod
     def new_session(cls, **session_spec):
         """New session for 'session_spec'"""
@@ -506,24 +619,6 @@ class RestHandler:
 class RequestsHandler(RestHandler):
     """Using requests (client is to bring in the dependency)"""
 
-    _is_usable = None
-
-    @classmethod
-    def is_usable(cls):
-        if cls._is_usable is None:
-            try:
-                import requests
-                import urllib3
-
-                # Silence logs, we use runez' more sophisticated logging setup
-                LogManager.silence(requests, urllib3)
-                cls._is_usable = True
-
-            except ImportError:  # pragma: no cover
-                cls._is_usable = False
-
-        return cls._is_usable
-
     @classmethod
     def default_retry(cls):
         import urllib3
@@ -601,12 +696,13 @@ class RestClient:
 
     handler = RequestsHandler
 
-    def __init__(self, base_url=None, headers=None, timeout=30, user_agent=None, handler=None, session=None, **session_spec):
+    def __init__(self, base_url=None, headers=None, timeout=30, cache=None, user_agent=None, handler=None, session=None, **session_spec):
         """
         Args:
             base_url (str | None): Base url of remote REST server
             headers (dict | None): Default headers to use
             timeout (int): Default timeout in seconds
+            cache (CacheWrapper): Optional local cache to use
             user_agent (str | None): User-Agent to use for outgoing calls coming from this client (default: handler.user_agent())
             handler: Optional: override default handler
             session: Optional: override getting handler.new_session()
@@ -614,12 +710,10 @@ class RestClient:
         self.base_url = base_url
         self.headers = dict(headers) if headers else {}
         self.timeout = timeout
+        self.cache_wrapper = cache
         if handler:
             self.handler = handler
 
-        if not self.handler or not self.handler.is_usable():
-            raise Exception("RestClient handler '%s' is not usable" % self.handler)
-
         self.user_agent = user_agent or self.handler.user_agent()
         self.session = session or self.handler.new_session(**session_spec)
         if self.user_agent:
@@ -651,54 +745,46 @@ class RestClient:
         """
         return urljoin(self.base_url, url)
 
-    def decompress(self, url, destination, simplify=False, fatal=True, logger=UNSET, dryrun=UNSET, params=None, headers=None):
+    def decompress(self, url, destination, simplify=False, fatal=True, logger=UNSET, dryrun=UNSET, **kwargs):
         """
         Args:
             url (str): URL of .tar.gz to unpack (may be absolute, or relative to self.base_url)
             destination (str | Path): Path to local folder where to untar url
             simplify (bool): If True and source has only one sub-folder, extract that one sub-folder to destination
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            dryrun (bool): Optionally override current dryrun setting
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
+            **kwargs: Passed through to underlying client
 
         Returns:
             (RestResponse): Response from underlying call
         """
+        _, _, actual_url = self._decomposed_checksum_url(url)
         destination = to_path(destination).absolute()
         with TempFolder():
-            tarball_path = to_path(os.path.basename(url)).absolute()
-            response = self.download(url, tarball_path, fatal=fatal, logger=logger, dryrun=dryrun, params=params, headers=headers)
+            tarball_path = to_path(os.path.basename(actual_url)).absolute()
+            response = self.download(url, tarball_path, fatal=fatal, logger=logger, dryrun=dryrun, **kwargs)
             if response.ok:
                 decompress(tarball_path, destination, simplify=simplify, fatal=fatal, logger=logger, dryrun=dryrun)
 
             return response
 
-    def download(self, url, destination, fatal=True, logger=UNSET, dryrun=UNSET, params=None, headers=None):
+    def download(self, url, destination, fatal=True, logger=UNSET, dryrun=UNSET, **kwargs):
         """
         Args:
             url (str): URL of resource to download (may be absolute, or relative to self.base_url)
                        Use #sha256=... or #sha512=... at the end of the url to ensure content is validated against given checksum
             destination (str | Path): Path to local file where to store the download
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            dryrun (bool): Optionally override current dryrun setting
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
+            **kwargs: Passed through to underlying client
 
         Returns:
             (RestResponse): Response from underlying call
         """
-        hash_algo = None
-        hash_checksum = None
-        m = re.search(r"#(md5|sha(1|256|512))=([a-f0-9]+)", url)
-        if m and m.end(0) == len(url):
-            hash_algo = m.group(1)
-            hash_checksum = m.group(3)
-            url = url[:m.start(0)]
-
-        response = self._get_response("GET", url, fatal, logger, dryrun=dryrun, params=params, headers=headers, action="download")
+        hash_algo, hash_checksum, url = self._decomposed_checksum_url(url)
+        response = self._get_response("GET", url, fatal, logger, dryrun=dryrun, action="download", **kwargs)
         if response.ok and not _R.resolved_dryrun(dryrun):
             destination = to_path(destination)
             ensure_folder(destination.parent, fatal=fatal, logger=None)
@@ -714,135 +800,127 @@ class RestClient:
 
         return response
 
-    def get_response(self, url, fatal=False, logger=False, params=None, headers=None):
+    def get_response(self, url, fatal=False, logger=False, **kwargs):
         """
         Args:
             url (str): Remote URL (may be absolute, or relative to self.base_url)
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            **kwargs: Passed through to underlying client
 
         Returns:
             (RestResponse): Response from underlying call
         """
-        return self._get_response("GET", url, fatal, logger, params=params, headers=headers)
+        return self._get_response("GET", url, fatal, logger, **kwargs)
 
-    def delete(self, url, fatal=True, logger=UNSET, dryrun=UNSET, params=None, headers=None):
+    def delete(self, url, fatal=True, logger=UNSET, dryrun=UNSET, **kwargs):
         """Same as underlying .delete(), but respecting 'dryrun' mode
 
         Args:
             url (str): URL to query (can be absolute, or relative to self.base_url)
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            dryrun (bool): Optionally override current dryrun setting
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
+            **kwargs: Passed through to underlying client
 
         Returns:
             (RestResponse): Response from underlying call
         """
-        return self._get_response("DELETE", url, fatal, logger, dryrun=dryrun, params=params, headers=headers)
+        return self._get_response("DELETE", url, fatal, logger, dryrun=dryrun, **kwargs)
 
-    def get(self, url, fatal=False, logger=False, params=None, headers=None):
+    def get(self, url, fatal=False, logger=False, **kwargs):
         """
         Args:
             url (str): Remote URL (may be absolute, or relative to self.base_url)
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            **kwargs: Passed through to underlying client
 
         Returns:
             (dict | None): Deserialized .json() from response, if available
         """
-        response = self.get_response(url, fatal=fatal, logger=logger, params=params, headers=headers)
+        response = self.get_response(url, fatal=fatal, logger=logger, **kwargs)
         if response.ok:
             return response.json()
 
-    def head(self, url, fatal=False, logger=False, params=None, headers=None):
+    def head(self, url, fatal=False, logger=False, **kwargs):
         """
         Args:
             url (str): URL to query (can be absolute, or relative to self.base_url)
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            **kwargs: Passed through to underlying client
 
         Returns:
             (RestResponse): Response from underlying call
         """
-        return self._get_response("HEAD", url, fatal, logger, params=params, headers=headers)
+        return self._get_response("HEAD", url, fatal, logger, **kwargs)
 
-    def post(
-        self, url, fatal=True, logger=UNSET, dryrun=UNSET, params=None, headers=None, data=None, json=None, files=None, filepaths=None
-    ):
+    def post(self, url, fatal=True, logger=UNSET, dryrun=UNSET, data=None, json=None, files=None, filepaths=None, **kwargs):
         """Same as underlying .post(), but respecting 'dryrun' mode
 
         Args:
             url (str): URL to query (can be absolute, or relative to self.base_url)
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            dryrun (bool): Optionally override current dryrun setting
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
             data (dict | tuple | bytes | file | None): Data to send in the body
             json: (optional) json to send in the body
             files (dict | None): File-like-objects for multipart encoding upload.
             filepaths (dict[str, Path] | None): File-like-objects for multipart encoding upload.
+            **kwargs: Passed through to underlying client
 
         Returns:
             (RestResponse): Response from underlying call
         """
         state = DataState.wrapped(dryrun, data, json, files, filepaths)
-        return self._get_response("POST", url, fatal, logger, dryrun=dryrun, params=params, headers=headers, state=state)
+        return self._get_response("POST", url, fatal, logger, dryrun=dryrun, state=state, **kwargs)
 
-    def purge(self, url, fatal=True, logger=UNSET, dryrun=UNSET, params=None, headers=None):
+    def purge(self, url, fatal=True, logger=UNSET, dryrun=UNSET, **kwargs):
         """Same as underlying .purge(), but respecting 'dryrun' mode
 
         Args:
             url (str): URL to query (can be absolute, or relative to self.base_url)
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            dryrun (bool): Optionally override current dryrun setting
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
+            **kwargs: Passed through to underlying client
 
         Returns:
             (RestResponse): Response from underlying call
         """
-        return self._get_response("PURGE", url, fatal, logger, dryrun=dryrun, params=params, headers=headers)
+        return self._get_response("PURGE", url, fatal, logger, dryrun=dryrun, **kwargs)
 
-    def put(self, url, fatal=True, logger=UNSET, dryrun=UNSET, params=None, headers=None, data=None, json=None, files=None, filepaths=None):
+    def put(self, url, fatal=True, logger=UNSET, dryrun=UNSET, data=None, json=None, files=None, filepaths=None, **kwargs):
         """
         Args:
             url (str): URL to query (can be absolute, or relative to self.base_url)
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            dryrun (bool): Optionally override current dryrun setting
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
             data (dict | tuple | bytes | file | None): Data to send in the body
             json: (optional) json to send in the body
             files (dict | None): File-like-objects for multipart encoding upload.
             filepaths (dict[str, Path] | None): File-like-objects for multipart encoding upload.
+            **kwargs: Passed through to underlying client
 
         Returns:
             (RestResponse): Response from underlying call
         """
         state = DataState.wrapped(dryrun, data, json, files, filepaths)
-        return self._get_response("PUT", url, fatal, logger, dryrun=dryrun, params=params, headers=headers, state=state)
+        return self._get_response("PUT", url, fatal, logger, dryrun=dryrun, state=state, **kwargs)
 
-    def url_exists(self, url, logger=False):
+    def url_exists(self, url, logger=False, **kwargs):
         """
         Args:
             url (str): URL to query (can be absolute, or relative to self.base_url)
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
+            **kwargs: Passed through to underlying client
 
         Returns:
             (bool): True if remote URL exists (ie: not a 404)
         """
-        response = self.head(url, logger=logger)
+        response = self.head(url, logger=logger, **kwargs)
         return bool(response and response.ok)
 
     def mock(self, specs):
@@ -870,6 +948,42 @@ class RestClient:
 
         return MockWrapper(self.handler, self.base_url, specs)
 
+    @staticmethod
+    def std_diskcache(directory=UNSET, default_expire=UNSET, size_limit=UNSET):
+        """
+        Convenience method to obtain a diskcache with good defaults, callers must require diskcache (not provided by runez)
+        This can be used as an example cache setup
+
+        Args:
+            directory (str | Path | None): Directory where to store the cache
+            default_expire (int | float | str): Default expiration time in seconds
+            size_limit (int | float | str): Size limit for this cache
+
+        Returns:
+            (CacheWrapper): Object wrapping this cache
+        """
+        try:
+            from diskcache import Cache
+
+            if directory is UNSET:
+                directory = None
+                if not DEV.current_test():
+                    # By default, do not use ~/.cache for test runs (diskcache will default to a temp dir)
+                    pymm = "py%s" % joined(sys.version_info[:2], delimiter="")
+                    directory = CacheWrapper.cache_base_path(suffix=pymm)
+
+            if default_expire is UNSET:
+                default_expire = CacheWrapper.default_expire
+
+            if size_limit is UNSET:
+                size_limit = _R.lc.rm.to_bytesize(CacheWrapper.size_limit)
+
+            cache_backend = Cache(directory=directory or None, size_limit=size_limit)
+            return CacheWrapper(cache_backend, directory, default_expire, size_limit)
+
+        except ImportError:
+            return None
+
     def _protected_get(self, method, absolute_url, keyword_args):
         try:
             return self.handler.raw_response(self.session, method, absolute_url, **keyword_args)
@@ -879,18 +993,31 @@ class RestClient:
 
         raise ForbiddenHttpError(absolute_url)
 
-    def _get_response(self, method, url, fatal, logger, dryrun=False, params=None, headers=None, state=None, action=None):
+    @classmethod
+    def _decomposed_checksum_url(cls, url):
+        regex = getattr(cls, "_checksum_regex", None)
+        if regex is None:
+            regex = cls._checksum_regex = re.compile(r"#(md5|sha(1|256|512))=([a-f0-9]+)")
+
+        m = regex.search(url)
+        if m and m.end(0) == len(url):
+            hash_algo = m.group(1)
+            hash_checksum = m.group(3)
+            return hash_algo, hash_checksum, url[: m.start(0)]
+
+        return None, None, url
+
+    def _get_response(self, method, url, fatal, logger, dryrun=False, state=None, action=None, **kwargs):
         """
         Args:
             method (str): Underlying method to call
             url (str): Remote URL (may be absolute, or relative to self.base_url)
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-            dryrun (bool): Optionally override current dryrun setting
-            params (dict | None): Key/value pairs for query string
-            headers (dict | None): Optional Headers specific to this request
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
             state (DataState | None): For PUT/POST requests
             action (str | None): Action to refer to in dryrun message (default: method)
+            **kwargs: Passed through to underlying client
 
         Returns:
             (RestResponse): Response from underlying call
@@ -898,14 +1025,29 @@ class RestClient:
         absolute_url = self.full_url(url)
         message = "%s %s" % (action or method, absolute_url)
         if _R.hdry(dryrun, logger, message):
-            return RestResponse(method, absolute_url, MockResponse(200, dict(message="dryrun %s" % message)))
+            return RestResponse(method, absolute_url, MockResponse(200, {"message": "dryrun %s" % message}))
+
+        cache_key = cache_expire = None
+        if self.cache_wrapper is not None:
+            cache_expire = kwargs.pop("expire", UNSET)
+            cache_key = self.cache_wrapper.cache_key(absolute_url, params=kwargs.get("params"))
+            if method in ("PURGE", "DELETE"):
+                self.cache_wrapper.delete(cache_key)
+
+            elif self.cache_wrapper.is_cachable_method(method):
+                response = self.cache_wrapper.get(cache_key)
+                if response is not None:
+                    return response
 
         full_headers = self.headers
+        headers = kwargs.get("headers")
         if headers:
             full_headers = dict(full_headers)
             full_headers.update(headers)
 
-        keyword_args = dict(headers=full_headers, params=params, timeout=self.timeout)
+        keyword_args = dict(kwargs)
+        keyword_args["headers"] = full_headers
+        keyword_args.setdefault("timeout", self.timeout)
         if state is not None:
             state.complete(keyword_args)
 
@@ -919,6 +1061,9 @@ class RestClient:
 
                 _R.hlog(logger, msg)
 
+            if cache_key is not None and response.ok and self.cache_wrapper.is_cachable_method(method):
+                self.cache_wrapper.set(cache_key, response, expire=cache_expire)
+
             return response
 
         finally:
