@@ -5,7 +5,6 @@ Convenience logging setup
 import faulthandler
 import logging
 import os
-import re
 import signal
 import sys
 import threading
@@ -17,12 +16,27 @@ from runez.ascii import AsciiAnimation
 from runez.convert import to_bytesize, to_int
 from runez.date import local_timezone, represented_duration
 from runez.file import parent_folder
-from runez.system import _R, abort, cached_property, decode, DEV, find_caller, flattened, quoted, short, stringified, uncolored
-from runez.system import LOG, py_mimic, Slotted, SYS_INFO, ThreadGlobalContext, UNSET, WINDOWS
-
+from runez.system import (
+    _R,
+    abort_if,
+    cached_property,
+    decode,
+    DEV,
+    find_caller,
+    flattened,
+    LOG,
+    py_mimic,
+    quoted,
+    short,
+    Slotted,
+    stringified,
+    SYS_INFO,
+    ThreadGlobalContext,
+    uncolored,
+    UNSET,
+)
 
 ORIGINAL_CF = logging.currentframe
-RE_FORMAT_MARKERS = re.compile(r"{([a-z]\w*)}", re.IGNORECASE)
 
 
 def formatted(message, *args, **named_values):
@@ -71,7 +85,6 @@ class ProgressHandler(logging.Handler):
 
 
 class ProgressBar:
-
     def __init__(self, iterable=None, total=None, columns=8, frames=" ▏▎▍▌▋▊▉"):
         self.columns = columns
         self.frames = frames
@@ -125,26 +138,25 @@ class ProgressBar:
 
     def rendered(self):
         """Called in spinner thread (lock already acquired)"""
-        if self.n is None:
+        if self.n is None or not self.total:
             return None
 
-        if self.total:
-            percent = max(int(round(100.0 * self.n / self.total)), 0)
-            blanks = 0
-            if percent >= 100:
-                percent = 100
-                bar = self.full_char * self.columns
+        percent = max(round(100.0 * self.n / self.total), 0)
+        blanks = 0
+        if percent >= 100:
+            percent = 100
+            bar = self.full_char * self.columns
 
-            else:
-                full_chars = int(percent / self.per_char)
-                bar = self.full_char * full_chars
-                blanks = self.columns - full_chars
-                if self.per_frame:
-                    blanks -= 1
-                    fi = int((percent - full_chars * self.per_char) / self.per_frame)
-                    bar += self.frames[fi]
+        else:
+            full_chars = int(percent / self.per_char)
+            bar = self.full_char * full_chars
+            blanks = self.columns - full_chars
+            if self.per_frame:
+                blanks -= 1
+                fi = int((percent - full_chars * self.per_char) / self.per_frame)
+                bar += self.frames[fi]
 
-            return "%s%s%s%%" % (bar, self.blank_char * blanks, percent)
+        return "%s%s%s%%" % (bar, self.blank_char * blanks, percent)
 
     def _remove_parent(self, parent):
         if parent is self.parent:
@@ -155,7 +167,6 @@ class ProgressBar:
 
 
 class _SpinnerComponent:
-
     def __init__(self, fps, source, color, adapter=None):
         self.adapter = adapter
         self.source = source  # type: callable
@@ -194,7 +205,6 @@ class _SpinnerComponent:
 
 
 class _SpinnerState:
-
     def __init__(self, parent, frames, max_columns, message_color, progress_color, spinner_color):
         """
         Args:
@@ -531,7 +541,7 @@ class LogSpec(Slotted):
         return bool(self.locations)
 
     def _props(self, **additional):
-        r = dict(argv=self.argv, pid=self.pid)
+        r = {"argv": self.argv, "pid": self.pid}
         r.update(self.to_dict())
         r.update(additional)
         return r
@@ -603,7 +613,7 @@ class _ContextFilter(logging.Filter):
 
 
 def default_log_locations():
-    if WINDOWS:  # pragma: no cover
+    if SYS_INFO.platform_id.is_windows:  # pragma: no cover
         return [os.path.join("{dev}", "log", "{basename}")]
 
     return ["{dev}/log/{basename}", "/logs/{appname}/{basename}", "/var/log/{basename}"]
@@ -639,13 +649,9 @@ class Timeit:
             (callable): Decorated function
         """
         if self.__func__:
-            self.__enter__()
-            try:
+            with self:
                 return self.__func__(*args, **kwargs)
 
-            finally:
-                self.__exit__()
-
         # We've been used as a decorator with args, and now we're called with the decorated function as argument
         self.__func__ = args[0]
         if not self.function_name:
@@ -657,7 +663,7 @@ class Timeit:
         self.start_time = time.time()
         return self
 
-    def __exit__(self, *_):
+    def __exit__(self, exc_type, exc_value, exc_traceback):
         msg = self.function_name
         if not msg:
             msg = "%s()" % find_caller()
@@ -668,7 +674,16 @@ class Timeit:
             elapsed = represented_duration(elapsed)
             msg = _R.colored(msg, self.color)
             elapsed = _R.colored(elapsed, self.color)
-            msg = self.fmt.format(function=msg, elapsed=elapsed)
+            if exc_value:
+                description = _R.colored("failed", "red")
+                if exc_type is not SystemExit:
+                    description += f": {exc_value}"
+
+                msg += f" {description} (ran for {elapsed})"
+
+            else:
+                msg = self.fmt.format(function=msg, elapsed=elapsed)
+
             logger(msg)
 
 
@@ -702,7 +717,7 @@ class LogManager:
 
     # Spec defines how logs should be setup()
     # Best way to provide your spec is via: runez.log.setup(), for example:
-    #   runez.log.setup(rotate="size:50m")
+    # >>> runez.log.setup(rotate="size:50m")
     spec = LogSpec(_default_spec)
 
     # Thread-local / global context
@@ -716,12 +731,12 @@ class LogManager:
 
     # Below fields should be read-only for outside users, do not modify these
     debug = False
-    console_handler = None  # type: Optional[logging.StreamHandler]
-    file_handler = None  # type: Optional[logging.FileHandler] # File we're currently logging to (if any)
-    handlers = None  # type: Optional[List[logging.Handler]]
-    tracer = None  # type: Optional[TraceHandler]
-    used_formats = None  # type: Optional[str]
-    faulthandler_signum = None  # type: Optional[int]
+    console_handler: Optional[logging.StreamHandler] = None
+    file_handler: Optional[logging.FileHandler] = None  # File we're currently logging to (if any)
+    handlers: Optional[List[logging.Handler]] = None
+    tracer: Optional[TraceHandler] = None
+    used_formats: Optional[str] = None
+    faulthandler_signum: Optional[int] = None
     trace_env_var = "TRACE_DEBUG"
 
     # Convenience decorator/context logging how long a function or section of code took to run
@@ -874,9 +889,7 @@ class LogManager:
                     message = "\n%s\n%s\n%s\n\n" % (bars, message, bars)
 
                 message = _R.colored(message, "red")
-                if allow_root is None:
-                    abort(message)
-
+                abort_if(allow_root is None, message)
                 LOG.warning(message)
 
     @staticmethod
@@ -952,7 +965,7 @@ class LogManager:
         return any(marker in used_formats for marker in flattened(markers, split=" "))
 
     @classmethod
-    def enable_faulthandler(cls, signum=getattr(signal, "SIGUSR1", None)):
+    def enable_faulthandler(cls, signum=UNSET):
         """Enable dumping thread stack traces when specified signals are received, similar to java's handling of SIGQUIT
 
         Note: this must be called from the surviving process in case of daemonization.
@@ -960,6 +973,9 @@ class LogManager:
         Args:
             signum (int | None): Signal number to register for full thread stack dump (use None to disable)
         """
+        if signum is UNSET:
+            signum = getattr(signal, "SIGUSR1", None)
+
         with cls._lock:
             if not signum:
                 cls._disable_faulthandler()
@@ -970,8 +986,8 @@ class LogManager:
 
             cls.faulthandler_signum = signum
             dump_file = cls.file_handler.stream
-            faulthandler.enable(file=dump_file, all_threads=True)  # noqa
-            faulthandler.register(signum, file=dump_file, all_threads=True, chain=False)  # noqa
+            faulthandler.enable(file=dump_file, all_threads=True)
+            faulthandler.register(signum, file=dump_file, all_threads=True, chain=False)
 
     @classmethod
     def override_spec(cls, **settings):
@@ -1029,7 +1045,7 @@ class LogManager:
 
         Args:
             message (str | callable | None): Message to log
-            dryrun (bool): Optionally override current dryrun setting
+            dryrun (bool | UNSET | None): Optionally override current dryrun setting
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
 
         Returns:
@@ -1057,10 +1073,10 @@ class LogManager:
     def _auto_enable_progress_handler(cls):
         if cls.progress.is_running:
             if ProgressHandler not in logging.root.handlers:
-                logging.root.handlers.append(ProgressHandler)  # noqa
+                logging.root.handlers.append(ProgressHandler)
 
         elif ProgressHandler in logging.root.handlers:
-            logging.root.handlers.remove(ProgressHandler)  # noqa
+            logging.root.handlers.remove(ProgressHandler)
 
     @classmethod
     def _update_used_formats(cls):
@@ -1168,7 +1184,8 @@ class LogManager:
         logging.logProcesses = cls.is_using_format("%(process)")
         logging.logThreads = cls.is_using_format("%(thread) %(threadName)")
 
-        if not isinstance(logging.info, _LogWrap) and _R.getframe is not None:
+        getframe = getattr(sys, "_getframe", None)
+        if not isinstance(logging.info, _LogWrap) and getframe is not None:
             logging.critical = _LogWrap(logging.CRITICAL)
             logging.fatal = logging.critical
             logging.error = _LogWrap(logging.ERROR)
@@ -1189,11 +1206,12 @@ class _LogWrap:
 
     @staticmethod
     def log(level, msg, *args, **kwargs):
+        getframe = getattr(sys, "_getframe", None)
         offset = kwargs.pop("_stack_offset", 1)
-        name = _R.getframe(offset).f_globals.get("__name__")
+        name = getframe(offset).f_globals.get("__name__")
         logger = logging.getLogger(name)
         try:
-            logging.currentframe = lambda: _R.getframe(3 + offset)
+            logging.currentframe = lambda: getframe(3 + offset)
             logger.log(level, msg, *args, **kwargs)
 
         finally:
@@ -1241,7 +1259,7 @@ def _canonical_format(fmt):
 
 
 def _format_recursive(key, value, definitions, max_depth):
-    m = RE_FORMAT_MARKERS.search(value)
+    m = _R.lc.rx_format_markers.search(value)
     if not m:
         return value
 
@@ -1275,7 +1293,7 @@ def _formatted_text(text, props, strict=False, max_depth=3):
         return text
 
     definitions = {}
-    markers = RE_FORMAT_MARKERS.findall(text)
+    markers = _R.lc.rx_format_markers.findall(text)
     while markers:
         key = markers.pop()
         if key in definitions:
@@ -1286,13 +1304,13 @@ def _formatted_text(text, props, strict=False, max_depth=3):
             return None
 
         val = stringified(val) if val is not None else "{%s}" % key
-        markers.extend(m for m in RE_FORMAT_MARKERS.findall(val) if m not in definitions)
+        markers.extend(m for m in _R.lc.rx_format_markers.findall(val) if m not in definitions)
         definitions[key] = val
 
     if not max_depth or not isinstance(max_depth, int) or max_depth <= 0:
         return text
 
-    result = dict((k, _format_recursive(k, v, definitions, max_depth)) for k, v in definitions.items())
+    result = {k: _format_recursive(k, v, definitions, max_depth) for k, v in definitions.items()}
     return text.format(**result)
 
 
@@ -1349,7 +1367,6 @@ def _get_file_handler(location, rotate, rotate_count):
 
 
 class _WrappedInstanceFunction:
-
     def __init__(self, function, instance):
         self.__func__ = function
         self.instance = instance
