@@ -16,17 +16,7 @@ from io import BytesIO
 from select import select
 
 from runez.convert import parsed_tabular, to_int
-from runez.system import _R, abort, cached_property, decode, flattened, quoted, resolved_path, short, SYS_INFO, uncolored
-from runez.system import UNSET, WINDOWS
-
-
-DEFAULT_INSTRUCTIONS = {
-    "darwin": "run: `brew install {program}`",
-    "linux": "run: `apt install {program}`",
-}
-PS_FOLLOW = {
-    "tmux": ("tmux", "display-message", "-p", "#{client_pid}"),
-}
+from runez.system import _R, abort, cached_property, decode, flattened, quoted, resolved_path, short, SYS_INFO, uncolored, UNSET
 
 
 class PsInfo:
@@ -93,6 +83,10 @@ class PsInfo:
 
         return cmd
 
+    @cached_property
+    def ps_follow(self):
+        return {"tmux": ("tmux", "display-message", "-p", "#{client_pid}")}
+
     @cached_property
     def followed_parent(self):
         """
@@ -100,7 +94,7 @@ class PsInfo:
             (PsInfo | None): Parent process info (if any), special processes like tmux are followed through
         """
         if self.parent and self.parent.ppid == 1:
-            follow_command = PS_FOLLOW.get(self.parent.cmd_basename)
+            follow_command = self.ps_follow.get(self.parent.cmd_basename)
             if follow_command:
                 r = run(*follow_command, dryrun=False, fatal=False, logger=None)
                 if r.succeeded:
@@ -155,14 +149,21 @@ class PsInfo:
 
     def parent_list(self, follow=True):
         """
-        Args:
-            follow (bool): If True, try and follow special processes like tmux
+        Parameters
+        ----------
+        follow: bool
+            If True, try and follow special processes like tmux
 
-        Returns:
-            (list[PsInfo]): List of parent processes
+        Returns
+        -------
+        list[PsInfo]
+            List of parent processes
         """
         p = self.followed_parent if follow else self.parent
-        return [p] + p.parent_list(follow=follow) if p else []
+        if not p:
+            return []
+
+        return [p, *p.parent_list(follow=follow)]
 
 
 def auto_shellify(args):
@@ -180,10 +181,10 @@ def check_pid(pid):
     Returns:
         (bool): True if process with pid exists
     """
-    if not pid:  # No support for kill pid 0, as that is not the intent of this function, and it's not cross platform
+    if not pid:  # No support for kill pid 0, as that is not the intent of this function, and it's not cross-platform
         return False
 
-    if WINDOWS:  # pragma: no cover
+    if SYS_INFO.platform_id.is_windows:  # pragma: no cover
         import ctypes
 
         kernel32 = ctypes.windll.kernel32
@@ -227,12 +228,12 @@ def daemonize():
 def is_executable(path):
     """
     Args:
-        path (str | None): Path to file
+        path (str | pathlib.Path | None): Path to file
 
     Returns:
         (bool): True if file exists and is executable
     """
-    if WINDOWS:  # pragma: no cover
+    if SYS_INFO.platform_id.is_windows:  # pragma: no cover
         return bool(_windows_exe(path))
 
     return path and os.path.isfile(path) and os.access(path, os.X_OK)
@@ -241,10 +242,10 @@ def is_executable(path):
 def make_executable(path, fatal=True, logger=UNSET, dryrun=UNSET):
     """
     Args:
-        path (str): chmod file with 'path' as executable
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        path (str | pathlib.Path): chmod file with 'path' as executable
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -259,7 +260,7 @@ def make_executable(path, fatal=True, logger=UNSET, dryrun=UNSET):
         return abort("%s does not exist, can't make it executable" % short(path), return_value=-1, fatal=fatal, logger=logger)
 
     try:
-        os.chmod(path, 0o755)  # nosec
+        os.chmod(path, 0o755)  # noqa: S103
         _R.hlog(logger, "Made '%s' executable" % short(path))
         return 1
 
@@ -268,18 +269,28 @@ def make_executable(path, fatal=True, logger=UNSET, dryrun=UNSET):
 
 
 def run(
-    program, *args, background=False, fatal=True, logger=UNSET, dryrun=UNSET, short_exe=UNSET,
-    passthrough=False, path_env=None, strip="\r\n",
-    stdout=subprocess.PIPE, stderr=subprocess.PIPE, **popen_args
+    program,
+    *args,
+    background=False,
+    fatal=True,
+    logger=UNSET,
+    dryrun=UNSET,
+    short_exe=UNSET,
+    passthrough=False,
+    path_env=None,
+    strip="\r\n",
+    stdout=subprocess.PIPE,
+    stderr=subprocess.PIPE,
+    **popen_args,
 ):
     """Run 'program' with 'args'
     Args:
         program (str | pathlib.Path): Program to run (full path, or basename)
         *args: Command line args to call 'program' with
         background (bool): When True, background the spawned process (detach from console and current process)
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
         short_exe (str | bool | None): Try to log a compact representation of executable
         passthrough (bool | file | None): If True-ish, pass-through stderr/stdout in addition to capturing it
                                           as well as 'passthrough' itself if it has a write() function
@@ -337,7 +348,7 @@ def run(
 
         fatal = False  # pragma: no cover, non-fatal mode in background process (there is no more console etc to report anything)
 
-    with _WrappedArgs([full_path] + args) as wrapped_args:
+    with _WrappedArgs([full_path, *args]) as wrapped_args:
         try:
             p, out, err = _run_popen(wrapped_args, popen_args, passthrough, fatal, stdout, stderr)
             result.output = decode(out, strip=strip)
@@ -356,16 +367,11 @@ def run(
 
         if fatal and result.exit_code:
             base_message = "%s exited with code %s" % (short(program), result.exit_code)
-            if passthrough and (result.output or result.error):
-                exception = _R.abort_exception(override=fatal)
-                if exception is SystemExit:
-                    raise SystemExit(result.exit_code)
-
-                if isinstance(exception, type) and issubclass(exception, BaseException):
-                    raise exception(base_message)
+            if passthrough:
+                abort(base_message, code=result.exit_code, exc_info=result.exc_info, fatal=fatal, logger=abort_logger)
 
             message = []
-            if abort_logger is not None and not passthrough:
+            if abort_logger is not None:
                 # Log full output, unless user explicitly turned it off
                 message.append("Run failed: %s" % description)
                 if result.error:
@@ -374,8 +380,8 @@ def run(
                 if result.output:
                     message.append("\nstdout:\n%s" % result.output)
 
-            message.append(base_message)
-            abort("\n".join(message), code=result.exit_code, exc_info=result.exc_info, fatal=fatal, logger=abort_logger)
+            message = _R.lc.rm.joined(message, base_message, delimiter="\n")
+            abort(message, code=result.exit_code, exc_info=result.exc_info, fatal=fatal, logger=abort_logger)
 
         if background:
             os._exit(result.exit_code)  # pragma: no cover, simply exit forked process (don't go back to caller)
@@ -388,7 +394,7 @@ def shell(*args, fatal=False, logger=False, dryrun=False):
 
     Args:
         *args: Program and arguments to run
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
         dryrun (bool): Override current dryrun setting
 
@@ -415,11 +421,14 @@ class RunAudit:
         self.popen_args = popen_args
         self.dryrun = False  # Was this a dryrun?
 
+    def __repr__(self):
+        return self.run_description()
+
     @staticmethod
     def shortened_program(program, args):
-        if program and args:
+        if program:
             base = os.path.basename(program)
-            if base in ("python", "python3"):
+            if args and base in ("python", "python3"):
                 if args[0] == "-m":
                     return args[1], args[2:]
 
@@ -520,14 +529,14 @@ def which(program, ignore_own_venv=False):
     program = str(program)
     if os.path.basename(program) != program:
         program = resolved_path(program)
-        if WINDOWS:  # pragma: no cover
+        if SYS_INFO.platform_id.is_windows:  # pragma: no cover
             return _windows_exe(program)
 
         return program if is_executable(program) else None
 
     for p in os.environ.get("PATH", "").split(os.pathsep):
         fp = os.path.join(p, program)
-        if WINDOWS:  # pragma: no cover
+        if SYS_INFO.platform_id.is_windows:  # pragma: no cover
             fp = _windows_exe(fp)
 
         if fp and (not ignore_own_venv or not SYS_INFO.venv_bin_folder or not fp.startswith(SYS_INFO.venv_bin_folder)):
@@ -541,27 +550,27 @@ def which(program, ignore_own_venv=False):
     return None
 
 
-def require_installed(program, instructions=None, platform=sys.platform):
+def require_installed(program, instructions=None, platform=None):
     """Raise an exception if 'program' is not available on PATH, show instructions on how to install it
 
     Args:
-        program (str): Program to check
+        program (str | pathlib.Path): Program to check
         instructions (str | dict): Short instructions letting user know how to get `program` installed, example: `run: brew install foo`
                                    Extra convenience, specify:
-                                   - None if `program` can simply be install via `brew install <program>`
-                                   - A word (without spaces) to refer to "usual" package (brew on OSX, apt on Linux etc)
-                                   - A dict with instructions per `sys.platform`
-        platform (str | None): Override sys.platform (for testing instructions rendering)
+                                   - None if `program` can simply be installed via `brew install <program>`
+                                   - A word (without spaces) to refer to "usual" package (brew on OSX, apt on Linux etc.)
+                                   - A dict with per-platform instructions
+        platform (str | None): Override SYS_INFO.platform (for testing instructions rendering)
 
     Returns:
         (bool): True if installed, False otherwise (when fatal=False)
     """
     if which(program) is None:
         if not instructions:
-            instructions = DEFAULT_INSTRUCTIONS
+            instructions = {"macos": "run: `brew install {program}`", "linux": "run: `apt install {program}`"}
 
         if isinstance(instructions, dict):
-            instructions = _install_instructions(instructions, platform)
+            instructions = _install_instructions(instructions, platform or SYS_INFO.platform_id.platform)
 
         message = "{program} is not installed"
         if instructions:
@@ -622,7 +631,7 @@ def _read_data(fd, length=1024):
 def _run_popen(args, popen_args, passthrough, fatal, stdout, stderr):
     """Run subprocess.Popen(), capturing output accordingly"""
     if not passthrough:
-        p = subprocess.Popen(args, stdout=stdout, stderr=stderr, **popen_args)  # nosec
+        p = subprocess.Popen(args, stdout=stdout, stderr=stderr, **popen_args)  # noqa: S603
         if fatal is None and stdout is None and stderr is None:
             return p, None, None  # Don't wait on spawned process
 
@@ -632,17 +641,21 @@ def _run_popen(args, popen_args, passthrough, fatal, stdout, stderr):
     # Capture output, but also let it pass-through as-is to the terminal
     stdout_r, stdout_w = pty.openpty()
     stderr_r, stderr_w = pty.openpty()
-    stdout_buffer = BytesIO()
-    stderr_buffer = BytesIO()
     term_size = struct.pack("HHHH", SYS_INFO.terminal.lines, SYS_INFO.terminal.columns, 0, 0)
     for fd in (stdout_r, stdout_w, stderr_r, stderr_w):
         fcntl.ioctl(fd, termios.TIOCSWINSZ, term_size)
 
     passthrough = getattr(passthrough, "stream", passthrough)  # Convenience support for things like logging handlers
-    if not hasattr(passthrough, "write"):
-        passthrough = None
+    if hasattr(passthrough, "write"):
+        # Don't accumulate out to RAM if we're passing it through to a file
+        stdout_buffer = stderr_buffer = None
 
-    with subprocess.Popen(args, stdout=stdout_w, stderr=stderr_w, **popen_args) as p:  # nosec
+    else:
+        passthrough = None
+        stdout_buffer = BytesIO()
+        stderr_buffer = BytesIO()
+
+    with subprocess.Popen(args, stdout=stdout_w, stderr=stderr_w, **popen_args) as p:  # noqa: S603
         os.close(stdout_w)
         os.close(stderr_w)
         readable = [stdout_r, stderr_r]
@@ -670,9 +683,17 @@ def _run_popen(args, popen_args, passthrough, fatal, stdout, stderr):
 
                     readable.remove(fd)
 
+    sys.stdout.flush()
+    sys.stderr.flush()
     os.close(stdout_r)
     os.close(stderr_r)
-    return p, uncolored(decode(stdout_buffer.getvalue())), uncolored(decode(stderr_buffer.getvalue()))
+    if stdout_buffer:
+        stdout_buffer = uncolored(decode(stdout_buffer.getvalue()))
+
+    if stderr_buffer:
+        stderr_buffer = uncolored(decode(stderr_buffer.getvalue()))
+
+    return p, stdout_buffer, stderr_buffer
 
 
 def _safe_write(target, data, flush=None):
@@ -682,7 +703,7 @@ def _safe_write(target, data, flush=None):
             if flush is not None:
                 flush.flush()
 
-        except Exception:  # nosec, don't consider run crashed if one of the channels we're passing through is failing
+        except Exception:  # noqa: S110, don't consider run crashed if one of the channels we're passing through is failing
             pass
 
 
@@ -709,13 +730,14 @@ class _WrappedArgs:
 
     def __enter__(self):
         args = self.args
-        if not WINDOWS and "PYCHARM_HOSTED" in os.environ and len(args) > 1 and "python" in args[0] and args[1][:2] in ("-m", "-X", "-c"):
+        needs_wrap = not SYS_INFO.platform_id.is_windows and "PYCHARM_HOSTED" in os.environ and len(args) > 1
+        if needs_wrap and "python" in args[0] and args[1][:2] in ("-m", "-X", "-c"):
             self.tmp_folder = os.path.realpath(tempfile.mkdtemp())
             wrapper = os.path.join(self.tmp_folder, "pydev-wrapper.sh")
             with open(wrapper, "wt") as fh:
                 fh.write('exec "$@"\n')
 
-            args = ["/bin/sh", wrapper] + args
+            args = ["/bin/sh", wrapper, *args]
 
         return args
 
