@@ -6,11 +6,11 @@ import collections
 import datetime
 import io
 import json
+from typing import ClassVar
 
 from runez.file import ensure_folder, parent_folder
 from runez.system import _R, abort, is_basetype, is_iterable, LOG, resolved_path, short, stringified, UNSET
 
-
 K_INDENTED_SEPARATORS = (",", ": ")
 K_COMPACT_SEPARATORS = (", ", ": ")
 
@@ -40,7 +40,7 @@ def with_behavior(strict=UNSET, extras=UNSET, hook=UNSET):
     Returns:
         (type): Internal temp class (compatible with `Serializable` metaclass) indicating how to handle Serializable type checking
     """
-    return BaseMetaInjector("_MBehavior", tuple(), {"behavior": DefaultBehavior(strict=strict, extras=extras, hook=hook)})
+    return BaseMetaInjector("_MBehavior", (), {"behavior": DefaultBehavior(strict=strict, extras=extras, hook=hook)})
 
 
 def is_serializable_descendant(base):
@@ -103,7 +103,7 @@ class DefaultBehavior:
         if extras is UNSET:
             extras = self.extras
 
-        self.strict = _to_callable(strict, fallback=_R.schema().ValidationException)
+        self.strict = _to_callable(strict, fallback=_R.lc.rm_schema.ValidationException)
         self.hook = _to_callable(hook)  # Called if provided at the end of ClassMetaDescription initialization
         self.ignored_extras = None  # Internal, populated if given `extras` is a `tuple(callable, list)`
 
@@ -196,21 +196,23 @@ def json_sanitized(value, stringify=stringified, dt=str, none=False):
     if value is None or is_basetype(value):
         return value
 
-    if hasattr(value, "to_dict"):
+    if isinstance(value, Serializable):
+        value = value.to_dict(stringify=stringify, dt=dt, none=none)
+
+    elif hasattr(value, "to_dict"):
         value = value.to_dict()
 
     elif isinstance(value, set):
         value = sorted(value)
 
     if isinstance(value, dict):
-        return dict(
-            (
-                json_sanitized(none if k is None and isinstance(none, str) else k, stringify=stringify, dt=dt, none=none),
-                json_sanitized(v, stringify=stringify, dt=dt, none=none),
+        return {
+            json_sanitized(none if k is None and isinstance(none, str) else k, stringify=stringify, dt=dt, none=none): json_sanitized(
+                v, stringify=stringify, dt=dt, none=none
             )
             for k, v in value.items()
             if none or (k is not None and v is not None)
-        )
+        }
 
     if is_iterable(value):
         return [json_sanitized(v, stringify=stringify, dt=dt, none=none) for v in value]
@@ -288,8 +290,8 @@ def scan_all_attributes(cls):
 class SerializableDescendants:
     """Tracks all descendants of Serializable, with at least one attribute defined"""
 
-    by_name = {}  # Tracks by class name only, last imported class wins
-    by_qualified_name = {}  # Tracks by full qualified name (won't be any conflicts)
+    by_name: ClassVar = {}  # Tracks by class name only, last imported class wins
+    by_qualified_name: ClassVar = {}  # Tracks by full qualified name (won't be any conflicts)
 
     @classmethod
     def descendant_with_name(cls, name):
@@ -358,11 +360,12 @@ class ClassMetaDescription:
                     self.properties.append(key)
                     continue
 
-                schema_type = _R.schema().determined_schema_type(value, required=False)
+                schema_module = _R.lc.rm_schema
+                schema_type = schema_module.determined_schema_type(value, required=False)
                 if schema_type is not None:
-                    if isinstance(schema_type, _R.schema().UniqueIdentifier):
+                    if isinstance(schema_type, schema_module.UniqueIdentifier):
                         if self.unique_identifier:
-                            raise _R.schema().ValidationException(
+                            raise schema_module.ValidationException(
                                 "Multiple unique ids specified for %s: %s and %s"
                                 % (self.qualified_name, self.unique_identifier, schema_type)
                             )
@@ -372,7 +375,7 @@ class ClassMetaDescription:
                     self.attributes[key] = schema_type
                     by_type[schema_type.__class__].append(key)
 
-        self._by_type = dict((k, sorted(v)) for k, v in by_type.items())  # Sorted to make things deterministic
+        self._by_type = {k: sorted(v) for k, v in by_type.items()}  # Sorted to make things deterministic
         if self.attributes:
             SerializableDescendants.register(self)
 
@@ -396,7 +399,7 @@ class ClassMetaDescription:
         """
         Args:
             data (dict): Raw data, coming for example from a json file
-            source (str | None): Optional, description of source where 'data' came from
+            source (str | pathlib.Path | None): Optional, description of source where 'data' came from
 
         Returns:
             (cls): Deserialized object
@@ -411,7 +414,7 @@ class ClassMetaDescription:
         Args:
             obj (Serializable): Object to populate
             data (dict): Raw data, coming for example from a json file
-            source (str | None): Optional, description of source where 'data' came from
+            source (str | pathlib.Path | None): Optional, description of source where 'data' came from
         """
         given = {} if data is None else dict(data)  # Copy of data
         for name, schema_type in self.attributes.items():
@@ -463,7 +466,8 @@ class ClassMetaDescription:
         Returns:
             (list): Tuple of attribute names and values for which values differ between `obj1` and `obj2`
         """
-        assert obj1._meta is self and obj2._meta is self
+        assert obj1._meta is self
+        assert obj2._meta is self
         result = []
         for key in self.attributes:
             v1 = getattr(obj1, key)
@@ -543,11 +547,7 @@ class Serializable:
 
     def __eq__(self, other):
         if other is not None and other.__class__ is self.__class__:
-            for name in self._meta.attributes:
-                if not hasattr(other, name) or getattr(self, name) != getattr(other, name):
-                    return False
-
-            return True
+            return not any(not hasattr(other, x) or getattr(self, x) != getattr(other, x) for x in self._meta.attributes)
 
     def __ne__(self, other):
         return not (self == other)
@@ -559,9 +559,9 @@ class Serializable:
     def from_json(cls, path, default=None, fatal=False, logger=False):
         """
         Args:
-            path (str): Path to json file
+            path (str | pathlib.Path): Path to json file
             default (dict | list | str | None): Default if file is not present, or can't be deserialized
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
 
         Returns:
@@ -577,7 +577,7 @@ class Serializable:
         """
         Args:
             data (dict): Raw data, coming for example from a json file
-            source (str | None): Optional, description of source where 'data' came from
+            source (str | pathlib.Path | None): Optional, description of source where 'data' came from
 
         Returns:
             (cls): Deserialized object
@@ -588,7 +588,7 @@ class Serializable:
         """
         Args:
             data (dict): Raw data, coming for example from a json file
-            source (str | None): Optional, description of source where 'data' came from
+            source (str | pathlib.Path | None): Optional, description of source where 'data' came from
             merge (bool): If True, add `data` to existing fields
         """
         if data is not None:
@@ -619,7 +619,7 @@ class Serializable:
         Returns:
             (dict): This object serialized to a dict
         """
-        raw = dict((name, getattr(self, name)) for name in self._meta.attributes)
+        raw = {name: getattr(self, name) for name in self._meta.attributes}
         return json_sanitized(raw, stringify=stringify, dt=dt, none=none)
 
 
@@ -628,7 +628,7 @@ def from_json(value, default=None, fatal=False, logger=False):
     Args:
         value (str): Value to deserialize
         default (dict | list | str | None): Default returned if value can't be deserialized
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
 
     Returns:
@@ -653,7 +653,7 @@ def read_json(path, default=None, fatal=False, logger=False):
     Args:
         path (str | pathlib.Path | None): Path to file to deserialize
         default (dict | list | str | None): Default returned if file is not present, or can't be deserialized
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
 
     Returns:
@@ -695,7 +695,7 @@ def save_json(data, path, stringify=stringified, dt=str, none=False, indent=2, s
     """
     Args:
         data (object | None): Data to serialize and save
-        path (str | None): Path to file where to save
+        path (str | pathlib.Path | None): Path to file where to save
         stringify (callable | None): Function to use to stringify non-builtin types
         dt (callable | None): Function to use to stringify dates
         none (str | bool): States how to treat `None` keys/values
@@ -704,9 +704,9 @@ def save_json(data, path, stringify=stringified, dt=str, none=False, indent=2, s
                            - True: No filtering, keep `None` keys/values as-is
         indent (int | None): Indentation to use, if None: use compact (one line) mode
         sort_keys (bool): Whether keys should be sorted
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        dryrun (bool): Optionally override current dryrun setting
+        dryrun (bool | UNSET | None): Optionally override current dryrun setting
 
     Returns:
         (int): In non-fatal mode, 1: successfully done, 0: was no-op, -1: failed
@@ -719,7 +719,10 @@ def save_json(data, path, stringify=stringified, dt=str, none=False, indent=2, s
         if _R.hdry(dryrun, logger, "save %s" % short(path)):
             return 1
 
-        ensure_folder(parent_folder(path), fatal=fatal, logger=None)
+        r = ensure_folder(parent_folder(path), fatal=fatal, logger=logger)
+        if r < 0:
+            return r
+
         data = json_sanitized(data, stringify=stringify, dt=dt, none=none)
         with open(path, "wt") as fh:
             json.dump(data, fh, indent=indent, sort_keys=sort_keys, separators=K_INDENTED_SEPARATORS if indent else K_COMPACT_SEPARATORS)
