@@ -13,15 +13,11 @@ import sys
 import threading
 import unicodedata
 from io import StringIO
-
+from typing import ClassVar
 
 ABORT_LOGGER = logging.error
 LOG = logging.getLogger("runez")
-RE_ANSI_ESCAPE = re.compile(r"\x1b(\[[;\d]*[A-Za-z]?)?")
-RE_SPACES = re.compile(r"[\s\n]+", re.MULTILINE)
 SYMBOLIC_TMP = "<tmp>"
-THREAD_LOCAL = threading.local()
-WINDOWS = sys.platform.startswith("win")
 
 
 class Undefined:
@@ -77,7 +73,7 @@ def abort(message, code=1, exc_info=None, return_value=None, fatal=True, logger=
         code (int): Exit code used when runez.system.AbortException is set to SystemExit
         exc_info (Exception): Exception info to pass on to logger
         return_value (Any): Value to return when `fatal` is not True
-        fatal (type | bool | None): True: abort execution, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
         logger (callable | bool | None): Logger to use, True to print(), None to disable log chatter
 
     Returns:
@@ -93,19 +89,33 @@ def abort(message, code=1, exc_info=None, return_value=None, fatal=True, logger=
         exception = _R.abort_exception(override=fatal)
         if exception is SystemExit:
             # Ensure message shown if we raise SystemExit
-            _show_abort_message(message, exc_info, logger or ABORT_LOGGER)
+            _show_abort_message(message, exc_info, fatal, logger or ABORT_LOGGER)
             raise SystemExit(code)
 
         if isinstance(exception, type) and issubclass(exception, BaseException):
-            _show_abort_message(message, exc_info, logger)
+            _show_abort_message(message, exc_info, fatal, logger)
             raise exception(message)
 
     elif fatal is not None:
-        _show_abort_message(message, exc_info, logger)
+        _show_abort_message(message, exc_info, fatal, logger)
 
     return return_value
 
 
+def abort_if(condition, message=None, code=1, exc_info=None, logger=UNSET):
+    """Abort if 'condition' is True-ish
+
+    Args:
+        condition (str | bool | None): Abort if True-ish
+        message (str): Message explaining why we're aborting (default: 'condition', which should be a string then)
+        code (int): Exit code used when runez.system.AbortException is set to SystemExit
+        exc_info (Exception): Exception info to pass on to logger
+        logger (callable | bool | None): Logger to use, True to print(), None to disable log chatter
+    """
+    if condition:
+        abort(_R.actual_message(message or condition), code=code, exc_info=exc_info, logger=logger)
+
+
 def py_mimic(target, source):
     """Make 'target' mimic python definition of 'source'
     Args:
@@ -250,11 +260,7 @@ def decode(value, strip=None):
         value = value.decode("utf-8")
 
     if strip:
-        if strip is True:
-            value = value.strip()
-
-        else:
-            value = value.strip(strip)
+        value = value.strip(strip if isinstance(strip, str) else None)
 
     return value
 
@@ -271,20 +277,20 @@ def find_caller(depth=2, maximum=1000, need_file=True, need_package=False, regex
     Returns:
         (_CallerInfo | None): Caller info, if any
     """
-    if _R.getframe is not None:
+    getframe = getattr(sys, "_getframe", None)
+    if getframe is not None:
         ignored = ("importlib", "pluggy", "runez")
         while not maximum or depth <= maximum:
             try:
-                f = _R.getframe(depth)
+                f = getframe(depth)
                 package = f.f_globals.get("__package__")
                 if package or not need_package:
                     name = f.f_globals.get("__name__")
                     top_level = package and package.partition(".")[0]
                     if name.endswith("__main__") or not top_level or (not top_level.startswith("_") and top_level not in ignored):
                         filepath = f.f_globals.get("__file__")
-                        if filepath or not need_file:
-                            if regex is None or regex.match(name):
-                                return _CallerInfo(f, depth, package, top_level, name, filepath)
+                        if (filepath or not need_file) and (regex is None or regex.match(name)):
+                            return _CallerInfo(f, depth, package, top_level, name, filepath)
 
                 depth = depth + 1
 
@@ -364,7 +370,7 @@ def get_version(mod, default="0.0.0", fatal=False, logger=False):
         mod (module | str): Module, or module name to find version for (pass either calling module, or its .__name__)
         default (str): Value to return if version determination fails
         logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
-        fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+        fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
 
     Returns:
         (str | None): Determined version
@@ -374,28 +380,17 @@ def get_version(mod, default="0.0.0", fatal=False, logger=False):
         name = mod.__name__
 
     if name and isinstance(name, str):
-        top_level = name.partition(".")[0] if isinstance(name, str) else name
+        top_level = name.partition(".")[0]
         last_exception = None
+        metadata_version = _metadata_version_function()
+        if metadata_version:
+            try:
+                v = metadata_version(top_level)
+                if v:
+                    return v
 
-        try:
-            from importlib import metadata
-
-            version = metadata.version(top_level)
-            if version:
-                return version
-
-        except (ImportError, Exception) as e:  # Python < 3.8
-            last_exception = e
-
-        try:
-            import pkg_resources
-
-            d = pkg_resources.get_distribution(top_level)
-            if d and d.version:
-                return d.version
-
-        except (ImportError, Exception) as e:
-            last_exception = e
+            except Exception as e:
+                last_exception = e
 
         m = sys.modules.get(name)
         if m is not None:
@@ -487,6 +482,29 @@ def joined(*args, delimiter=" ", keep_empty=False, strip=None, stringify=stringi
     return delimiter.join(args)
 
 
+def ltattr(self, other, *fields, t=None):
+    """Allows to avoid repeating implementation of __lt__, where fields of an object are simply compared in order
+    Missing fields (None) arbitrarily are considered < all others
+
+    Args:
+        self: Object issuing the comparison
+        other: Object 'self' is being compared to
+        t: If provided, 'other' must be an instance of given type 't'
+        *fields: Fields to examine
+
+    Returns:
+        (bool): True if there is one field 'x' such that obj1.x < obj2.x, and all previous fields are ==
+    """
+    if t is not None and not isinstance(other, t):
+        return NotImplemented
+
+    for field in fields:
+        f1 = getattr(self, field)
+        f2 = getattr(other, field)
+        if f1 != f2:
+            return f1 is None or (f2 is not None and f1 < f2)
+
+
 def quoted(*items, delimiter=" ", adapter=UNSET, keep_empty=True, strip=None, stringify=stringified, unique=False):
     """Quoted `items`, for those that contain whitespaces
 
@@ -545,7 +563,7 @@ def resolved_path(path, base=None):
 
     path = os.path.expanduser(path)
     if base and not os.path.isabs(path):
-        return os.path.join(resolved_path(base), path)
+        path = os.path.join(resolved_path(base), path)
 
     return os.path.abspath(path)
 
@@ -561,7 +579,7 @@ def short(value, size=UNSET, none="None", uncolor=False):
         (str): Leading part of text, with at most 'size' chars (when specified)
     """
     text = stringified(value, converter=_prettified, none=none).strip()
-    text = RE_SPACES.sub(" ", text)
+    text = _R.lc.rx_spaces.sub(" ", text)
     text = Anchored.short(text)
     if size is UNSET or isinstance(size, int) and size < 0:
         size = SYS_INFO.terminal.columns
@@ -570,10 +588,10 @@ def short(value, size=UNSET, none="None", uncolor=False):
         if uncolor and "\033" in text:
             clear_text = uncolored(text)
             if len(clear_text) > size:
-                text = "%s..." % clear_text[:size - 3]
+                text = "%s..." % clear_text[: size - 3]
 
         else:
-            text = "%s..." % text[:size - 3]
+            text = "%s..." % text[: size - 3]
 
     return text
 
@@ -586,7 +604,7 @@ def uncolored(text):
     Returns:
         (str): Text without any ANSI color rendering
     """
-    return RE_ANSI_ESCAPE.sub("", stringified(text))
+    return _R.lc.rx_ansi_escape.sub("", stringified(text))
 
 
 def wcswidth(text):
@@ -659,7 +677,7 @@ class AdaptedProperty:
         >>> assert my_object.width == 10
     """
 
-    __counter = [0]  # Simple counter for anonymous properties
+    __counter: ClassVar = [0]  # Simple counter for anonymous properties
 
     def __init__(self, validator=None, default=None, doc=None, caster=None, type=None):
         """
@@ -676,7 +694,8 @@ class AdaptedProperty:
         assert caster is None or type is None, "Can't accept both 'caster' and 'type' for AdaptedProperty, pick one"
         if callable(validator):
             # 'validator' is available when used as decorator of the form: @AdaptedProperty
-            assert caster is None and type is None, "'caster' and 'type' are not applicable to AdaptedProperty decorator"
+            assert caster is None, "'caster' is not applicable to AdaptedProperty decorator"
+            assert type is None, "'type' is not applicable to AdaptedProperty decorator"
             self.validator = validator
             self.key = "__%s" % validator.__name__
             py_mimic(self, validator)
@@ -694,7 +713,8 @@ class AdaptedProperty:
 
     def __call__(self, validator):
         """Called when used as decorator of the form: @AdaptedProperty(default=...)"""
-        assert self.caster is None and self.type is None, "'caster' and 'type' are not applicable to decorated properties"
+        assert self.caster is None, "'caster' is not applicable to decorated properties"
+        assert self.type is None, "'type' is not applicable to decorated properties"
         self.validator = validator
         self.key = "__%s" % validator.__name__
         py_mimic(self, validator)
@@ -727,7 +747,7 @@ class Anchored:
     """
 
     _home = None
-    _paths = []  # Currently stacked anchored folders that can be simplified away, via short()
+    _paths: ClassVar = []  # Currently stacked anchored folders that can be simplified away, via short()
 
     def __init__(self, *folders):
         self.folders = folders
@@ -775,7 +795,7 @@ class Anchored:
             (str): Short form, using '~' if applicable
         """
         if cls._home is None:
-            cls._home = os.path.expanduser("~")
+            cls._home = "" if os.geteuid() == 0 else os.path.expanduser("~")
 
         text = stringified(text)
         if cls._paths:
@@ -783,7 +803,10 @@ class Anchored:
                 if p:
                     text = text.replace(p + os.path.sep, "")
 
-        return text.replace(cls._home, "~")
+        if cls._home:
+            text = text.replace(cls._home, "~")
+
+        return text
 
 
 class CapturedStream:
@@ -862,7 +885,7 @@ class CaptureOutput:
     'foo bar'
     """
 
-    _capture_stack = []  # Shared across all objects, tracks possibly nested CaptureOutput buffers
+    _capture_stack: ClassVar = []  # Shared across all objects, tracks possibly nested CaptureOutput buffers
 
     def __init__(self, stdout=True, stderr=True, anchors=None, dryrun=UNSET, seed_logging=False, trace=False):
         """Context manager allowing to temporarily grab stdout/stderr/log output.
@@ -916,7 +939,7 @@ class CaptureOutput:
             logging.root.addHandler(self.handler)
 
         if self.trace:
-            self.prior_trace = _R._runez_module().log.enable_trace(True)
+            self.prior_trace = _R.lc.rm.log.enable_trace(True)
 
         return self.tracked
 
@@ -935,7 +958,7 @@ class CaptureOutput:
             Anchored.pop(self.anchors)
 
         if self.trace:
-            _R._runez_module().log.enable_trace(self.prior_trace)
+            _R.lc.rm.log.enable_trace(self.prior_trace)
 
 
 class CurrentFolder:
@@ -962,6 +985,20 @@ class CurrentFolder:
             Anchored.pop(self.destination)
 
 
+class OverrideDryrun:
+    """Context manager to temporarily override dryrun mode"""
+
+    def __init__(self, dryrun):
+        self.dryrun = dryrun
+
+    def __enter__(self):
+        self.prior = _R.set_dryrun(self.dryrun)
+        return self.prior
+
+    def __exit__(self, *_):
+        _R.set_dryrun(self.prior)
+
+
 class TrackedOutput:
     """Track captured output"""
 
@@ -1109,7 +1146,7 @@ class Slotted:
     def fill_attributes(obj, named):
         """Allows to turn 'named' into named attributes, UNSET values allow to fall back to stated default at class level"""
         if isinstance(obj, type):
-            raise ValueError("extract_settings() called on class %s (should be instance)" % obj)
+            raise TypeError("extract_settings() called on class %s (should be instance)" % obj)
 
         for k, v in named.items():
             if k and not k.startswith("_"):
@@ -1130,17 +1167,13 @@ class Slotted:
 
     def __eq__(self, other):
         if isinstance(other, self.__class__):
-            for name in self.__slots__:
-                if getattr(self, name, None) != getattr(other, name, None):
-                    return False
-
-            return True
+            return all(getattr(self, x, None) == getattr(other, x, None) for x in self.__slots__)
 
     def _seed(self):
         """Seed initial fields"""
         defaults = self._get_defaults()
         if not isinstance(defaults, dict):
-            defaults = dict((k, defaults) for k in self.__slots__)
+            defaults = dict.fromkeys(self.__slots__, defaults)
 
         for name in self.__slots__:
             value = getattr(self, name, defaults.get(name))
@@ -1197,7 +1230,7 @@ class Slotted:
     def _values_from_object(self, obj):
         """dict: Optional hook to allow descendants to extract key/value pairs from an object"""
         if obj is not None:
-            return dict((k, getattr(obj, k, UNSET)) for k in self.__slots__)
+            return {k: getattr(obj, k, UNSET) for k in self.__slots__}
 
 
 class DevInfo:
@@ -1242,7 +1275,7 @@ class DevInfo:
     def venv_folder(self) -> str:
         """Path to current development venv, if we're running from one"""
         if SYS_INFO.venv_bin_folder:
-            return _R.find_parent_folder(sys.prefix, {"venv", ".venv", ".tox", "build"})
+            return _R.find_parent_folder(sys.prefix, {"venv", ".venv", ".virtualenvs", ".tox", "build"})
 
     def project_path(self, *relative_path) -> str:
         """Full path relative to development project we're currently running from (if any)"""
@@ -1281,7 +1314,7 @@ class PlatformId:
     subsystem: str = None  # Example: libc, musl (empty for macos/windows for now)
 
     default_subsystem = None  # Can this be auto-detected? (currently: users can optionally provide this, by setting this class field)
-    platform_archive_type = dict(linux="tar.gz", macos="tar.gz", windows="zip")
+    platform_archive_type: ClassVar = {"linux": "tar.gz", "macos": "tar.gz", "windows": "zip"}
     sys_include = None  # Most standard system include dirs, if any
 
     rx_base_path = None  # Regex identifying "base" libraries (present on any declination of this system)
@@ -1310,12 +1343,13 @@ class PlatformId:
             subsystem = self.determine_current_subsystem()
 
         self.subsystem = subsystem
-        base_paths = ["@rpath/.+"]
+        base_paths = []
         if self.is_using_libc:
-            # Similar to https://github.com/pypa/auditwheel/blob/master/auditwheel/policy/manylinux-policy.json
+            # Similar to https://github.com/pypa/auditwheel/tree/main/src/auditwheel/policy
             base_paths.append("libc.so.6")
             base_paths.append("libcrypt.so.1")
             base_paths.append("libdl.so.2")
+            base_paths.append("libexpat.so.1")
             base_paths.append("libm.so.6")
             base_paths.append("libnsl.so.1")
             base_paths.append("libpthread.so.0")
@@ -1331,10 +1365,10 @@ class PlatformId:
         elif self.is_macos:
             self.sys_include = ["/Library/Developer/CommandLineTools/SDKs/MacOSX.sdk/usr/include"]
             self.rx_sys_lib = re.compile(r"^/(usr/lib\d*|System/Library)/.+$")
+            base_paths.append("@(rpath|executable_path|loader_path)/.+")  # Count relative libs as base
             base_paths.append(r"/usr/lib/libSystem\.B\.dylib")
 
         elif self.is_windows:
-            # TODO: add windows support
             base_paths.append(r"C:/Windows/System32/.*")
 
         self.rx_base_path = re.compile(r"^(%s)$" % joined(base_paths, delimiter="|"))
@@ -1355,7 +1389,7 @@ class PlatformId:
             short_form (bool): If True, return the short form (ie: no "tar.")
 
         Returns:
-            (str): Canonical form of 'extension', or default extension for self.platform
+            (str): Canonical form of 'extension', or default extension for 'self.platform'
         """
         if not extension:
             extension = self.platform_archive_type.get(self.platform) or "tar.gz"
@@ -1398,16 +1432,12 @@ class PlatformId:
 
     def is_base_lib(self, *paths):
         """Does one of the given 'paths' match a base library? (present on any declination of this system)"""
-        for path in paths:
-            if path:
-                if self.rx_base_path.match(str(path)):
-                    return True
+        return any(p and self.rx_base_path.match(str(p)) for p in paths)
 
     def is_system_lib(self, *paths):
         """Does one of the given 'paths' match a system library? (ie: installed in a system folder, not /usr/local and such)"""
-        for path in paths:
-            if path and self.rx_sys_lib and self.rx_sys_lib.match(str(path)):
-                return True
+        if self.rx_sys_lib:
+            return any(p and self.rx_sys_lib.match(str(p)) for p in paths)
 
     @property
     def is_using_libc(self):
@@ -1431,11 +1461,25 @@ class PlatformId:
 
         return platform.machine()
 
+    @staticmethod
+    def canonical_platform(name):
+        name = name and name.lower()
+        if name == "darwin":
+            # Using 'macos' to differentiate from iOS (which may be detected in the future)
+            name = "macos"
+
+        elif name.startswith("linux"):
+            name = "linux"
+
+        if name.startswith("win"):
+            name = "windows"
+
+        return name
+
     def determine_current_platform(self):
         import platform
 
-        p = platform.system().lower()
-        return "macos" if p == "darwin" else p
+        return self.canonical_platform(platform.system())
 
     def determine_current_subsystem(self):
         return self.default_subsystem and self.default_subsystem.get(self.platform)
@@ -1446,7 +1490,7 @@ class PlatformInfo:
 
     def __init__(self, text=None):
         if text is None:
-            text = "Windows" if WINDOWS else _R.shell_output("uname", "-msrp")
+            text = "Windows" if SYS_INFO.platform_id.is_windows else _R.lc.rm.shell("uname", "-msrp")
 
         self.os_name = text or "unknown-os"
         self.os_version = None
@@ -1473,9 +1517,9 @@ class SystemInfo:
     @cached_property
     def current_process(self):
         """Info on currently running process"""
-        return _R._runez_module().PsInfo()
+        return _R.lc.rm.PsInfo()
 
-    def diagnostics(self, argv=UNSET, exe=True, platform=True, prefix=UNSET, term=UNSET, userid=UNSET, version=UNSET, via=" ⚡ "):
+    def diagnostics(self, argv=UNSET, exe=True, platform=True, term=UNSET, userid=UNSET, version=UNSET, via=" ⚡ "):
         """Usable by runez.render.PrettyTable.two_column_diagnostics()"""
         if platform:
             yield "platform", "%s [%s]" % (_R.colored(self.platform_id, "bold"), self.platform_info)
@@ -1492,6 +1536,8 @@ class SystemInfo:
 
         if exe:
             yield "sys.executable", sys.executable
+            if self.is_running_in_venv:
+                yield "invoker python", self.invoker_python
 
         if via:
             process_list = self.current_process.parent_list()
@@ -1501,8 +1547,19 @@ class SystemInfo:
         if _R.rdefault(argv, "diagnostics" not in sys.argv):
             yield "sys.argv", quoted(sys.argv)
 
-        if _R.rdefault(prefix, SYS_INFO.venv_bin_folder):
-            yield "sys.prefix", sys.prefix
+    @cached_property
+    def invoker_python(self):
+        """The python that is either currently running us, or that created the venv we're running from"""
+        import platform
+        from pathlib import Path
+
+        from runez.pyenv import PythonInstallation, PythonSimpleInspection
+
+        path = Path(sys.executable)
+        real_exe = path.resolve()
+        inspection = PythonSimpleInspection(version=".".join(str(s) for s in sys.version_info[:3]), machine=platform.machine())
+        PythonSimpleInspection.register(path, real_exe, inspection)
+        return PythonInstallation(real_exe)
 
     @cached_property
     def is_running_in_docker(self):
@@ -1512,7 +1569,7 @@ class SystemInfo:
 
         try:
             with open("/proc/1/cgroup") as fh:
-                regex = re.compile(r"docker|lxc|kubepod", re.IGNORECASE)
+                regex = re.compile(r"docker|buildkit|lxc|kubepod", re.IGNORECASE)
                 for line in fh:
                     if line and regex.search(line):
                         return True
@@ -1522,6 +1579,11 @@ class SystemInfo:
 
         return False
 
+    @cached_property
+    def is_running_in_venv(self):
+        """Are we currently running from a virtual environment?"""
+        return sys.prefix != sys.base_prefix
+
     @cached_property
     def platform_id(self):
         """Simplified platform identification for portable programs, example: linux-x86_64, or macos-arm64"""
@@ -1568,7 +1630,7 @@ class SystemInfo:
     @cached_property
     def venv_bin_folder(self):
         """Path to current venv/bin folder, if we're running from a virtual environment"""
-        if sys.prefix != sys.base_prefix:
+        if self.is_running_in_venv:
             return os.path.join(sys.prefix, "bin")
 
     def venv_bin_path(self, name):
@@ -1592,7 +1654,7 @@ class TempArgv:
         self.old_argv = sys.argv
 
     def __enter__(self):
-        sys.argv = [self.exe] + self.args
+        sys.argv = [self.exe, *self.args]
 
     def __exit__(self, *_):
         sys.argv = self.old_argv
@@ -1654,7 +1716,7 @@ class TerminalInfo:
         Returns:
             (int | None): Current number of columns, determined dynamically
         """
-        return _R.to_int(_R.shell_output("tput", "cols"))
+        return _R.lc.rm.to_int(_R.lc.rm.shell("tput", "cols"))
 
     @staticmethod
     def get_lines():
@@ -1662,7 +1724,7 @@ class TerminalInfo:
         Returns:
             (int | None): Current number of lines, determined dynamically
         """
-        return _R.to_int(_R.shell_output("tput", "lines"))
+        return _R.lc.rm.to_int(_R.lc.rm.shell("tput", "lines"))
 
     @staticmethod
     def get_size(default_columns=160, default_lines=25):
@@ -1670,8 +1732,8 @@ class TerminalInfo:
         Returns:
             (int, int): Current number of rows and columns of current terminal, if available
         """
-        cols = _R.to_int(os.environ.get("COLUMNS"))
-        lines = _R.to_int(os.environ.get("LINES"))
+        cols = _R.lc.rm.to_int(os.environ.get("COLUMNS"))
+        lines = _R.lc.rm.to_int(os.environ.get("LINES"))
         if not cols or not lines:
             size = shutil.get_terminal_size(fallback=(0, 0))
             cols = size.columns
@@ -1836,7 +1898,7 @@ class ThreadGlobalContext:
 
     def _ensure_threadlocal(self):
         if self._tpayload is None:
-            self._tpayload = THREAD_LOCAL
+            self._tpayload = threading.local()
 
         if not hasattr(self._tpayload, "log_context"):
             self._tpayload.log_context = {}
@@ -1850,9 +1912,151 @@ class ThreadGlobalContext:
             self._gpayload = values or {}
 
 
+class UnitRepresentation:
+    def __init__(self, base=1000, prefixes="KMGTP"):
+        """
+        Args:
+            base (int): Base to represent it in (example: 1024 for bytes, 1000 for bits)
+            prefixes (str): Prefixes to use per power (kilo, mega, giga, tera, peta, ...)
+        """
+        self.base = base
+        self.prefixes = prefixes
+
+    def unit_exponent(self, unit, default=None):
+        try:
+            return 0 if not unit else self.prefixes.index(unit.upper()) + 1
+
+        except ValueError:
+            return default
+
+    def unitized(self, value, base, unit):
+        """
+        Args:
+            value (int | float): Value to expand
+            base (int): Base to use (usually 1024)
+            unit (str): Given unit
+
+        Returns:
+            Deduced value (example: "1k" becomes 1000)
+        """
+        exponent = self.unit_exponent(unit)
+        if exponent is not None:
+            return round(value * (base**exponent))
+
+    def represented(self, size, base=None, delimiter="", unit="", exponent=0):
+        """
+        Args:
+            size (int | float): Size to represent
+            base (int): Base to represent it in (example: 1024 for bytes, 1000 for bits)
+            delimiter (str): Delimiter to use between number and units
+            unit (str): Unit symbol to use
+            exponent (int): Internal use
+
+        Returns:
+            (str): Human friendly byte size representation
+        """
+        if base is None:
+            base = self.base
+
+        if not base:
+            return "%g" % size
+
+        if size >= base and exponent < len(self.prefixes):
+            size = float(size) / base
+            return self.represented(size, base=base, delimiter=delimiter, unit=unit, exponent=exponent + 1)
+
+        if exponent == 0:
+            if unit:
+                return "%g%s%s" % (size, delimiter, unit)
+
+            return "%g" % size
+
+        fmt = "%.{precision}f".format(precision=0 if size > 9 else 1)
+        represented_size = fmt % size
+        if "." in represented_size:
+            represented_size = represented_size.strip("0").strip(".")
+
+        return "%s%s%s%s" % (represented_size, delimiter, self.prefixes[exponent - 1], unit)
+
+
+class _LazyCache:
+    @cached_property
+    def rm(self):
+        import runez
+
+        return runez
+
+    @cached_property
+    def rm_schema(self):
+        import runez.schema
+
+        return runez.schema
+
+    @cached_property
+    def rx_ansi_escape(self):
+        return re.compile(r"\x1b(\[[;\d]*[A-Za-z]?)?")
+
+    @cached_property
+    def rx_camel_cased_words(self):
+        return re.compile(r"(^[a-z]+|[A-Z](?:[a-z]+|[A-Z]*(?=[A-Z]|$)))")
+
+    @cached_property
+    def rx_date(self):
+        base_number = r"([-+]?[\d_]*\.?[\d_]*([eE][-+]?[\d_]+)?|[-+]?\.inf|[-+]?\.Inf|[-+]?\.INF|\.nan|\.NaN|\.NAN|0o[0-7]+|0x[\da-fA-F]+)"
+        base_date = (
+            r"((\d{1,4})[-/](\d\d?)[-/](\d{1,4})"
+            r"([Tt \t](\d\d?):(\d\d?):(\d\d?)(\.\d*)?"
+            r"([ \t]*(Z|[A-Z]{3}|[+-]\d\d?(:(\d\d?))?))?)?)"
+        )
+        return re.compile(rf"^\s*({base_number}|{base_date})\s*$")
+
+    @cached_property
+    def rx_duration(self):
+        return re.compile(r"^\s*(\d+[ywdhms]\s*)+$")
+
+    @cached_property
+    def rx_format_markers(self):
+        return re.compile(r"{([a-z]\w*)}", re.IGNORECASE)
+
+    @cached_property
+    def rx_spaces(self):
+        return re.compile(r"[\s\n]+", re.MULTILINE)
+
+    @cached_property
+    def rx_words(self):
+        return re.compile(r"[^\w]+")
+
+    @cached_property
+    def rx_tz(self):
+        return re.compile(r"\s*(Z|UTC|([+-]?\d\d):?(\d\d))\s*")
+
+    @cached_property
+    def rx_version(self):
+        # See https://peps.python.org/pep-0440/
+        return re.compile(
+            r"v?(?P<vtext>"
+            r"((?P<epoch>\d+)!)?"
+            r"(?P<main>\d+(\.\d+)*)"
+            r"([-_.]?(?P<pre>a|b|c|rc|alpha|beta|pre|preview)[-_.]?(?P<pre_num>\d+))?"
+            r"([-_.]?(?P<rel>post|rev|r)[-_.]?(?P<rel_num>\d*))?"
+            r"([-_.]?(?P<dev>dev)[-_.]?(?P<dev_num>\d*))?"
+            r"(\+(?P<local>[a-z0-9]+(?:[-_.][a-z0-9]+)*))?)"
+            r"(?P<ignored>.*)"
+        )
+
+    @cached_property
+    def true_tokens(self):
+        return {"on", "true", "y", "yes"}
+
+    @cached_property
+    def units(self):
+        return UnitRepresentation()
+
+
 class _R:
     """
-    Internal class to provide a late import of runez (after __init__ imported everything), and also holds some common stuff.
+    Internal class, used as a namespace
+    Provides a late import of runez (after __init__ imported everything), and also holds some common stuff.
     The name is intentionally short to avoid verbose/long lines calling it.
     _R stands for "runez, internal class"
 
@@ -1861,60 +2065,7 @@ class _R:
     - respecting any external modifications clients may have done (like: runez.DRYRUN = foo)
     """
 
-    getframe = getattr(sys, "_getframe", None)
-
-    _runez = None
-    _schema = None
-
-    @classmethod
-    def _runez_module(cls):
-        """Late-imported runez module"""
-        if cls._runez is None:
-            import runez
-
-            cls._runez = runez
-
-        return cls._runez
-
-    @staticmethod
-    def actual_message(message):
-        if callable(message):
-            message = message()  # Allow message to be late-called function
-
-        return message
-
-    @classmethod
-    def colored(cls, text, color, is_coloring=UNSET):
-        """Colored 'text' with 'color', 'is_coloring' can be used to override current coloring setting"""
-        return cls._runez_module().color.colored(text, color, is_coloring=is_coloring)
-
-    @classmethod
-    def filesize(cls, path):
-        return cls._runez_module().filesize(path)
-
-    @classmethod
-    def plural(cls, countable, singular=None):
-        return cls._runez_module().plural(countable, singular=singular)
-
-    @classmethod
-    def represented_bytesize(cls, size, unit="bytes"):
-        return cls._runez_module().represented_bytesize(size, unit=unit)
-
-    @classmethod
-    def shell_output(cls, program, *args):
-        return cls._runez_module().shell(program, *args)
-
-    @classmethod
-    def to_int(cls, text):
-        return cls._runez_module().to_int(text)
-
-    @staticmethod
-    def _schema_type_name(target):
-        meta = getattr(target, "meta", None)
-        if meta is not None:
-            return meta.name
-
-        return target.__class__.__name__
+    lc = _LazyCache()
 
     @classmethod
     def abort_exception(cls, override=None):
@@ -1922,7 +2073,17 @@ class _R:
         if isinstance(override, type) and issubclass(override, BaseException):
             return override
 
-        return cls._runez_module().system.AbortException
+        return cls.lc.rm.system.AbortException
+
+    @staticmethod
+    def actual_message(message):
+        """Allow message to be late-called function"""
+        return message() if callable(message) else message
+
+    @classmethod
+    def colored(cls, text, color, is_coloring=UNSET):
+        """Colored 'text' with 'color', 'is_coloring' can be used to override current coloring setting"""
+        return cls.lc.rm.color.colored(text, color, is_coloring=is_coloring)
 
     @classmethod
     def habort(cls, default, fatal, logger, message, exc_info=None):
@@ -1930,7 +2091,7 @@ class _R:
 
         Args:
             default: Default value to return, if 'fatal' is False
-            fatal (bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
+            fatal (type | bool | None): True: abort execution on failure, False: don't abort but log, None: don't abort, don't log
             logger (callable | bool | None): Logger to use, True to print(), False to trace(), None to disable log chatter
             message (str): Message explaining failure
             exc_info (Exception): Exception, if this comes from a try/except block
@@ -1938,9 +2099,7 @@ class _R:
         Returns:
             'default', if we didn't abort
         """
-        if fatal:
-            abort(_R.actual_message(message), exc_info=exc_info, fatal=fatal, logger=logger)
-
+        abort_if(fatal, message, exc_info=exc_info, logger=logger)
         _R.hlog(logger, message, exc_info=exc_info)
         return default
 
@@ -1983,7 +2142,7 @@ class _R:
             return
 
         if logger is UNSET:
-            logger = cls._runez_module().log.spec.default_logger
+            logger = cls.lc.rm.log.spec.default_logger
 
         if callable(logger):
             msg = _R.actual_message(message)
@@ -2003,7 +2162,7 @@ class _R:
         Returns:
             (bool): Same as runez.DRYRUN, but as a function (and with late import)
         """
-        return cls._runez_module().DRYRUN
+        return cls.lc.rm.DRYRUN
 
     @staticmethod
     def rdefault(value, default_value):
@@ -2019,24 +2178,14 @@ class _R:
             (bool): Resolved value for dryrun
         """
         if dryrun is UNSET:
-            return cls._runez_module().DRYRUN
+            return cls.lc.rm.DRYRUN
 
         return dryrun
 
-    @classmethod
-    def schema(cls):
-        """Late-imported schema"""
-        if cls._schema is None:
-            import runez.schema
-
-            cls._schema = runez.schema
-
-        return cls._schema
-
     @classmethod
     def serializable(cls):
         """Late-imported Serializable class"""
-        return cls._runez_module().Serializable
+        return cls.lc.rm.Serializable
 
     @classmethod
     def meta_description(cls, struct):
@@ -2047,7 +2196,7 @@ class _R:
         Returns:
             (runez.serialize.ClassMetaDescription): Meta object describing given 'struct'
         """
-        return cls._runez_module().serialize.ClassMetaDescription(struct.__class__)
+        return cls.lc.rm.serialize.ClassMetaDescription(struct.__class__)
 
     @classmethod
     def set_dryrun(cls, dryrun):
@@ -2059,7 +2208,7 @@ class _R:
         Returns:
             (bool): Old values for dryrun
         """
-        r = cls._runez_module()
+        r = cls.lc.rm
         old_dryrun = r.DRYRUN
         if dryrun is not UNSET:
             r.DRYRUN = bool(dryrun)
@@ -2072,7 +2221,7 @@ class _R:
         Args:
             message (str): Message to trace
         """
-        cls._runez_module().log.trace(message, *args)
+        cls.lc.rm.log.trace(message, *args)
 
     @staticmethod
     def find_parent_folder(path, basenames):
@@ -2097,12 +2246,7 @@ def _flatten(result, value, keep_empty, split, shellify, strip, transform, uniqu
 
                 return
 
-            if isinstance(split, str):
-                value = value.split(split)
-
-            else:
-                value = value.split()
-
+            value = value.split(split if isinstance(split, str) else None)
             if value:
                 _flatten(result, value, keep_empty, None, shellify, strip, transform, unique, none)
 
@@ -2181,19 +2325,17 @@ def _prettified(value):
         return "function '%s'" % value.__name__
 
 
-def _show_abort_message(message, exc_info, logger):
+def _show_abort_message(message, exc_info, fatal, logger):
     if logger is not None:
         if logging.root.handlers:
-            _R.hlog(logger, message, exc_info=exc_info)
+            _R.hlog(logger, message, exc_info=exc_info if fatal else None)
 
         else:
             sys.stderr.write("%s\n" % message)
 
 
 def _has_stream_handler():
-    for h in logging.root.handlers:
-        if isinstance(h, logging.StreamHandler) or getattr(h, "isolation", None) == 0:
-            return True
+    return any(isinstance(h, logging.StreamHandler) or getattr(h, "isolation", None) == 0 for h in logging.root.handlers)
 
 
 def _full_path(path, relative_path):
@@ -2203,11 +2345,21 @@ def _full_path(path, relative_path):
     return path
 
 
+def _metadata_version_function():
+    try:
+        from importlib.metadata import version  # requires py3.8+
+
+        return version
+
+    except ImportError:
+        return None
+
+
 def _validated_project_path(*paths):
     for path in paths:
         if path:
             path = os.path.dirname(path)
-            if os.path.exists(os.path.join(path, "setup.py")) or os.path.exists(os.path.join(path, "project.toml")):
+            if os.path.exists(os.path.join(path, "setup.py")) or os.path.exists(os.path.join(path, "pyproject.toml")):
                 return path
 
 
@@ -2228,19 +2380,16 @@ class _CallerInfo:
     def globals(self, prefix=None, private=False):
         """Iterate over the globals in caller"""
         for name, value in self.frame.f_globals.items():
-            if private or not name.startswith("_"):
-                if not prefix or name.startswith(prefix):
-                    yield name, value
+            if (private or not name.startswith("_")) and (not prefix or name.startswith(prefix)):
+                yield name, value
 
     @cached_property
     def basename(self):
-        if self.filepath:
-            return os.path.basename(self.filepath)
+        return os.path.basename(self.filepath) if self.filepath else None
 
     @cached_property
     def folder(self):
-        if self.filepath:
-            return os.path.dirname(self.filepath)
+        return os.path.dirname(self.filepath) if self.filepath else None
 
     @cached_property
     def function(self):
